@@ -27,6 +27,7 @@
 #include "Charting.h"
 #include "ChartExport.h"
 #include "XlsxXmlChartReader.h"
+#include "PptxXmlCommentsReader.h"
 
 #include <MsooXmlSchemas.h>
 #include <MsooXmlUtils.h>
@@ -55,11 +56,36 @@
 //#define HARDCODED_PRESENTATIONSTYLENAME
 
 #include <MsooXmlReader_p.h>
+#include <MsooXmlContentTypes.h>
 
 PptxShapeProperties::PptxShapeProperties()
 {
+    x = 0;
+    y = 0;
+    width = -1;
+    height = -1;
+    rot = 0;
+    isPlaceHolder = false;
 }
 
+PptxShapeProperties::PptxShapeProperties(const PptxShapeProperties &other)
+{
+    *this = other;
+}
+
+PptxShapeProperties& PptxShapeProperties::operator=(const PptxShapeProperties &other)
+{
+    x = other.x;
+    y = other.y;
+    width = other.width;
+    height = other.height;
+    rot = other.rot;
+    isPlaceHolder = other.isPlaceHolder;
+    return *this;
+}
+
+// -------------------
+
 PptxSlideProperties::PptxSlideProperties()
 {
 }
@@ -73,12 +99,24 @@ void PptxSlideProperties::clear()
 {
     qDeleteAll(shapes);
     shapes.clear();
+    shapesMap.clear();
 }
 
+// -------------------
+
 PptxPlaceholder::PptxPlaceholder()
 {
 }
 
+PptxPlaceholder::PptxPlaceholder(const PptxShapeProperties &other)
+ : x(EMU_TO_CM_STRING(other.x))
+ , y(EMU_TO_CM_STRING(other.y))
+ , width(EMU_TO_CM_STRING(other.width))
+ , height(EMU_TO_CM_STRING(other.height))
+{
+    kDebug() << x << y << width << height;
+}
+
 PptxPlaceholder::~PptxPlaceholder()
 {
 }
@@ -91,19 +129,75 @@ void PptxPlaceholder::writeAttributes(KoXmlWriter* writer)
     writer->addAttribute("svg:height", height);
 }
 
+// -------------------
+
+PptxSlideMasterListLevelTextStyle::PptxSlideMasterListLevelTextStyle()
+ : m_characterStyle(0)
+{
+}
+
+PptxSlideMasterListLevelTextStyle::~PptxSlideMasterListLevelTextStyle()
+{
+    delete m_characterStyle;
+}
+
+// -------------------
+
+PptxSlideMasterTextStyle::PptxSlideMasterTextStyle()
+ : m_listStyles(9)
+{
+}
+
+PptxSlideMasterTextStyle::~PptxSlideMasterTextStyle()
+{
+    clear();
+}
+
+void PptxSlideMasterTextStyle::clear()
+{
+    for (int i = 0; i < m_listStyles.size(); i++) {
+        delete m_listStyles[i];
+    }
+    m_listStyles.clear();
+}
+
+PptxSlideMasterListLevelTextStyle* PptxSlideMasterTextStyle::listStyle(uint level)
+{
+    if (level < 1 || level > (uint)m_listStyles.size())
+        return 0;
+    PptxSlideMasterListLevelTextStyle* result = m_listStyles.at(level - 1);
+    if (!result) {
+        m_listStyles[level - 1] = result = new PptxSlideMasterListLevelTextStyle;
+    }
+    return result;
+}
+
+// -------------------
+
 PptxSlideLayoutProperties::PptxSlideLayoutProperties()
 {
 }
 
 PptxSlideLayoutProperties::~PptxSlideLayoutProperties()
 {
+    qDeleteAll(shapes);
     qDeleteAll(placeholders);
 }
 
+// -------------------
+
 PptxSlideMasterPageProperties::PptxSlideMasterPageProperties()
 {
 }
 
+void PptxSlideMasterPageProperties::clear()
+{
+    titleStyle.clear();
+    bodyStyle.clear();
+    otherStyle.clear();
+    m_drawingPageProperties.clear();
+}
+
 void PptxSlideMasterPageProperties::addDrawingPageProperty(
     const QByteArray& property, const QByteArray& value)
 {
@@ -120,6 +214,17 @@ void PptxSlideMasterPageProperties::saveDrawingPageProperties(KoGenStyle* style)
     }
 }
 
+PptxSlideMasterTextStyle* PptxSlideMasterPageProperties::textStyle(const QString& style)
+{
+    if (style == QLatin1String("title") || style == QLatin1String("ctrTitle")) {
+        return &titleStyle;
+    }
+    else if (style == "body" || style == "subTitle") {
+        return &bodyStyle;
+    }
+    return &otherStyle;
+}
+
 // -------------------
 
 PptxXmlSlideReaderContext::PptxXmlSlideReaderContext(
@@ -128,12 +233,14 @@ PptxXmlSlideReaderContext::PptxXmlSlideReaderContext(
     PptxXmlSlideReader::Type _type, PptxSlideProperties* _slideProperties,
     PptxSlideLayoutProperties* _slideLayoutProperties,
     PptxSlideMasterPageProperties* _slideMasterPageProperties,
-    MSOOXML::MsooXmlRelationships& _relationships)
+    MSOOXML::MsooXmlRelationships& _relationships,
+    QMap<int, QString> _commentAuthors)
         : MSOOXML::MsooXmlReaderContext(&_relationships),
         import(&_import), path(_path), file(_file),
         slideNumber(_slideNumber), themes(&_themes), type(_type),
         slideProperties(_slideProperties), slideLayoutProperties(_slideLayoutProperties),
-        slideMasterPageProperties(_slideMasterPageProperties)
+        slideMasterPageProperties(_slideMasterPageProperties),
+        commentAuthors(_commentAuthors)
 {
 }
 
@@ -151,7 +258,19 @@ public:
     //! Used to index shapes in master slide when inheriting properties
     uint shapeNumber;
     QString qualifiedNameOfMainElement;
-    QString phType; //! set by read_ph()
+    QString phType; //!< set by read_ph()
+    QString phIdx; //!< set by read_ph()
+    QString phStyleId() const
+    {
+        if (!phType.isEmpty())
+            return phType;
+        return phIdx;
+    }
+
+    PptxSlideMasterTextStyle* currentSlideMasterTextStyle; //!< set by read_*Style()
+    //!set by read_t as true whenever some characters are copied to a textbox,
+    //!used to figure out if a shape is a placeholder or not
+    bool textBoxHasContent;
 };
 
 PptxXmlSlideReader::PptxXmlSlideReader(KoOdfWriters *writers)
@@ -181,6 +300,7 @@ KoFilter::ConversionStatus PptxXmlSlideReader::read(MSOOXML::MsooXmlReaderContex
 {
     m_context = dynamic_cast<PptxXmlSlideReaderContext*>(context);
     Q_ASSERT(m_context);
+    d->currentSlideMasterTextStyle = 0;
     switch (m_context->type) {
     case Slide:
         d->qualifiedNameOfMainElement = "p:sld";
@@ -379,95 +499,101 @@ KoFilter::ConversionStatus PptxXmlSlideReader::read_sldInternal()
 #endif
 
         // m_currentDrawStyle defined in "MsooXmlCommonReaderDrawingMLMethods.h"
-        m_currentDrawStyle = KoGenStyle(KoGenStyle::DrawingPageAutoStyle, "drawing-page"); // CASE #P109
-        m_currentDrawStyle.addProperty("presentation:background-visible", true);   // CASE #P111
-        m_currentDrawStyle.addProperty("presentation:background-objects-visible", true);   // CASE #P112
-        m_context->slideMasterPageProperties->saveDrawingPageProperties(&m_currentDrawStyle);
+        m_currentDrawStyle = new KoGenStyle(KoGenStyle::DrawingPageAutoStyle, "drawing-page"); // CASE #P109
+        m_currentDrawStyle->addProperty("presentation:background-visible", true);   // CASE #P111
+        m_currentDrawStyle->addProperty("presentation:background-objects-visible", true);   // CASE #P112
+        m_context->slideMasterPageProperties->saveDrawingPageProperties(m_currentDrawStyle);
     }
     else if (m_context->type == SlideMaster) {
-        m_currentDrawStyle = KoGenStyle(KoGenStyle::DrawingPageAutoStyle, "drawing-page");
-        m_currentDrawStyle.addProperty("presentation:visibility", "visible");
-        m_currentDrawStyle.addProperty("presentation:background-objects-visible", false);
+        m_currentDrawStyle = new KoGenStyle(KoGenStyle::DrawingPageAutoStyle, "drawing-page");
+        m_currentDrawStyle->addProperty("presentation:visibility", "visible");
+        m_currentDrawStyle->addProperty("presentation:background-objects-visible", false);
     }
     else if (m_context->type == SlideLayout) {
         m_currentPresentationPageLayoutStyle = KoGenStyle(KoGenStyle::PresentationPageLayoutStyle);
     }
 
-    {
-        MSOOXML::Utils::XmlWriteBuffer drawPageBuf; // buffer this draw:page, because we have to compute
-        // style before style name is known
+    MSOOXML::Utils::XmlWriteBuffer drawPageBuf; // buffer this draw:page, because we have to compute
+    // style before style name is known
 //        KoXmlWriter *origBody = body;
-        if (m_context->type == Slide) {
-            body = drawPageBuf.setWriter(body);
+    if (m_context->type == Slide) {
+        body = drawPageBuf.setWriter(body);
 //            body = new KoXmlWriter(&drawPageBuf, origBody->indentLevel()+1);
-        }
-        while (!atEnd()) {
-            readNext();
-            if (isStartElement()) {
-                TRY_READ_IF(cSld)
-                else if (m_context->type == SlideMaster && QUALIFIED_NAME_IS(txStyles)) {
-                    TRY_READ(txStyles)
-                }
+    }
+    while (!atEnd()) {
+        readNext();
+        if (isStartElement()) {
+            TRY_READ_IF(cSld)
+            else if (m_context->type == SlideMaster && QUALIFIED_NAME_IS(txStyles)) {
+                TRY_READ(txStyles)
+            }
 //! @todo add ELSE_WRONG_FORMAT
-            }
-            if (isEndElement()) {
-                if (d->qualifiedNameOfMainElement == qualifiedName()) {
-                    break;
-                }
+        }
+        if (isEndElement()) {
+            if (d->qualifiedNameOfMainElement == qualifiedName()) {
+                break;
             }
         }
+    }
 
-        if (m_context->type == Slide) {
-            body = drawPageBuf.originalWriter();
+    if (m_context->type == Slide) {
+        body = drawPageBuf.originalWriter();
 //            delete body;
 //            body = origBody;
-            {
-                body->startElement("draw:page"); // CASE #P300
-                //! @todo draw:master-page-name is hardcoded for now
-                body->addAttribute("draw:master-page-name", "Default"); // required; CASE #P301
-                //! @todo draw:name can be pulled out of docProps/app.xml (TitlesOfParts)
-                body->addAttribute("draw:name", QString("page%1").arg(m_context->slideNumber+1)); //optional; CASE #P303
-                body->addAttribute("draw:id", QString("pid%1").arg(m_context->slideNumber)); //optional; unique ID; CASE #P305, #P306
-                //! @todo presentation:use-date-time-name //optional; CASE #P304
+
+        body->startElement("draw:page"); // CASE #P300
+        //! @todo draw:master-page-name is hardcoded for now
+        body->addAttribute("draw:master-page-name", "Default"); // required; CASE #P301
+        //! @todo draw:name can be pulled out of docProps/app.xml (TitlesOfParts)
+        body->addAttribute("draw:name", i18n("Slide %1",m_context->slideNumber+1)); //optional; CASE #P303
+        body->addAttribute("draw:id", QString("pid%1").arg(m_context->slideNumber)); //optional; unique ID; CASE #P305, #P306
+        //! @todo presentation:use-date-time-name //optional; CASE #P304
 
 /*
-    <style:style style:family="drawing-page" style:name="a393">
-      <style:drawing-page-properties draw:fill="bitmap" draw:fill-image-name="a392" style:repeat="readerh".
-      presentation:visibility="visible" draw:background-size="border" presentation:background-objects-visible="true".
-      presentation:background-visible="true" presentation:display-header="false" presentation:display-footer="false" presentation:display-page-number="false".
-      presentation:display-date-time="false"/>
-    </style:style>
+<style:style style:family="drawing-page" style:name="a393">
+    <style:drawing-page-properties draw:fill="bitmap" draw:fill-image-name="a392" style:repeat="readerh".
+    presentation:visibility="visible" draw:background-size="border" presentation:background-objects-visible="true".
+    presentation:background-visible="true" presentation:display-header="false" presentation:display-footer="false" presentation:display-page-number="false".
+    presentation:display-date-time="false"/>
+</style:style>
 */
-                
-                const QString currentPageStyleName(mainStyles->insert(m_currentDrawStyle, "dp"));
-                body->addAttribute("draw:style-name", currentPageStyleName); // CASE #P302
-                kDebug() << "currentPageStyleName:" << currentPageStyleName;
+            
+        const QString currentPageStyleName(mainStyles->insert(*m_currentDrawStyle, "dp"));
+        body->addAttribute("draw:style-name", currentPageStyleName); // CASE #P302
+        kDebug() << "currentPageStyleName:" << currentPageStyleName;
 
-                if (!m_context->slideLayoutProperties->styleName.isEmpty()) {
-                    // CASE #P308
-                    kDebug() << "presentation:presentation-page-layout-name=" <<
-                                       m_context->slideLayoutProperties->styleName;
-                    body->addAttribute("presentation:presentation-page-layout-name",
-                                       m_context->slideLayoutProperties->styleName);
-                }
+        if (!m_context->slideLayoutProperties->pageLayoutStyleName.isEmpty()) {
+            // CASE #P308
+            kDebug() << "presentation:presentation-page-layout-name=" <<
+                                m_context->slideLayoutProperties->pageLayoutStyleName;
+            body->addAttribute("presentation:presentation-page-layout-name",
+                                m_context->slideLayoutProperties->pageLayoutStyleName);
+        }
 
 //                body->addCompleteElement(&drawPageBuf);
-                (void)drawPageBuf.releaseWriter();
-                body->endElement(); //draw:page
-            }
-        }
-        else if (m_context->type == SlideMaster) {
-            m_context->pageDrawStyleName = mainStyles->insert(m_currentDrawStyle, "dp");
-            mainStyles->markStyleForStylesXml(m_context->pageDrawStyleName);
-            kDebug() << "m_context->pageDrawStyleName:" << m_context->pageDrawStyleName
-                << "m_context->type:" << m_context->type;
+        (void)drawPageBuf.releaseWriter();
+
+        {
+            PptxXmlCommentsReader commentsReader(this);
+            const QString filepath = m_context->relationships->targetForType(m_context->path, m_context->file, MSOOXML::Relationships::comments);
+            PptxXmlCommentsReaderContext commentsContext;
+            commentsContext.authors = m_context->commentAuthors;
+            (void)m_context->import->loadAndParseDocument(&commentsReader, filepath, &commentsContext);
         }
+
+        body->endElement(); //draw:page
+    }
+    else if (m_context->type == SlideMaster) {
+        m_context->pageDrawStyleName = mainStyles->insert(*m_currentDrawStyle, "dp");
+        mainStyles->markStyleForStylesXml(m_context->pageDrawStyleName);
+        kDebug() << "m_context->pageDrawStyleName:" << m_context->pageDrawStyleName
+            << "m_context->type:" << m_context->type;
+    }
+    else if (m_context->type == SlideLayout) {
+        m_context->slideLayoutProperties->pageLayoutStyleName = mainStyles->insert(m_currentPresentationPageLayoutStyle);
+        kDebug() << "slideLayoutProperties->styleName:" << m_context->slideLayoutProperties->pageLayoutStyleName;
     }
 
-    if (m_context->type == SlideLayout) {
-        m_context->slideLayoutProperties->styleName = mainStyles->insert(m_currentPresentationPageLayoutStyle);
-        kDebug() << "slideLayoutProperties->styleName:" << m_context->slideLayoutProperties->styleName;
-    }
     return KoFilter::OK;
 }
 
@@ -529,6 +655,8 @@ KoFilter::ConversionStatus PptxXmlSlideReader::read_bodyStyle()
 
     QString currentListStyleName = "bodyList";
     m_currentListStyle = KoGenStyle(KoGenStyle::ListStyle, "list");
+    d->currentSlideMasterTextStyle = &m_context->slideMasterPageProperties->bodyStyle;
+    MSOOXML::Utils::Setter<PptxSlideMasterTextStyle*> currentSlideMasterTextStyleSetter(&d->currentSlideMasterTextStyle, 0);
 
     while (!atEnd()) {
         readNext();
@@ -573,6 +701,7 @@ KoFilter::ConversionStatus PptxXmlSlideReader::read_oleObj()
     TRY_READ_ATTR_WITH_NS(r, id);
     TRY_READ_ATTR_WITHOUT_NS(imgW);
     TRY_READ_ATTR_WITHOUT_NS(imgH);
+    TRY_READ_ATTR_WITHOUT_NS(progId);
 
     while (!atEnd()) {
         readNext();
@@ -583,20 +712,38 @@ KoFilter::ConversionStatus PptxXmlSlideReader::read_oleObj()
     }
 
     if (!r_id.isEmpty()) {
-        // The file which we get here is oble object, it should be unpacked.
-        // @todo : implement this
-        /*
-        body->startElement("draw:image");
-        const QString sourceName(m_context->relationships->target(m_context->path, m_context->file, r_id));
+        QString sourceName(m_context->relationships->target(m_context->path, m_context->file, r_id));
         if (sourceName.isEmpty()) {
             return KoFilter::FileNotFound;
         }
         QString destinationName;
-        RETURN_IF_ERROR( copyFile(sourceName, QLatin1String("Pictures/"), destinationName) )
-        addManifestEntryForPicturesDir();
+
+        body->startElement("draw:object-ole");
+        RETURN_IF_ERROR( copyFile(sourceName, "", destinationName))
         body->addAttribute("xlink:href", destinationName);
-        body->endElement(); //draw:image
-        */
+        body->endElement(); // draw:object-ole
+
+        if (progId == "Paint.Picture") {
+            body->startElement("draw:image");
+            RETURN_IF_ERROR( copyFile(sourceName, QLatin1String("Pictures/"), destinationName, true) )
+            addManifestEntryForPicturesDir();
+            body->addAttribute("xlink:href", destinationName);
+            body->addAttribute("xlink:show", "embed");
+            body->addAttribute("xlink:actuate", "onLoad");
+            body->endElement(); //draw:image
+        }
+        else if (progId == "Package") {
+            body->startElement("draw:plugin"); // The mimetype is not told by the ole container, this is best guess
+            RETURN_IF_ERROR( copyFile(sourceName, "", destinationName, true ))
+            body->addAttribute("xlink:href", destinationName);
+            body->endElement(); // draw:plugin
+        }
+        else if (progId.contains("AcroExch")) { // PDF
+            body->startElement("draw:object"); // The mimetype is not told by the ole container, this is best guess
+            RETURN_IF_ERROR( copyFile(sourceName, "", destinationName, true ))
+            body->addAttribute("xlink:href", destinationName);
+            body->endElement(); // draw:object
+        }
     }
 
     READ_EPILOGUE
@@ -629,6 +776,8 @@ KoFilter::ConversionStatus PptxXmlSlideReader::read_titleStyle()
 
     QString currentListStyleName = "titleList";
     m_currentListStyle = KoGenStyle(KoGenStyle::ListStyle, "list");
+    d->currentSlideMasterTextStyle = &m_context->slideMasterPageProperties->titleStyle;
+    MSOOXML::Utils::Setter<PptxSlideMasterTextStyle*> currentSlideMasterTextStyleSetter(&d->currentSlideMasterTextStyle, 0);
 
     while (!atEnd()) {
         readNext();
@@ -680,6 +829,8 @@ KoFilter::ConversionStatus PptxXmlSlideReader::read_otherStyle()
 
     QString currentListStyleName = "otherList";
     m_currentListStyle = KoGenStyle(KoGenStyle::ListStyle, "list");
+    d->currentSlideMasterTextStyle = &m_context->slideMasterPageProperties->otherStyle;
+    MSOOXML::Utils::Setter<PptxSlideMasterTextStyle*> currentSlideMasterTextStyleSetter(&d->currentSlideMasterTextStyle, 0);
 
     while (!atEnd()) {
         readNext();
@@ -832,16 +983,16 @@ KoFilter::ConversionStatus PptxXmlSlideReader::read_bgPr()
 
     if (!fillImageName.isEmpty()) {
         //! Setup slide's bitmap fill
-        m_currentDrawStyle.addProperty("draw:fill", "bitmap");
-        m_currentDrawStyle.addProperty("draw:fill-image-name", fillImageName);
+        m_currentDrawStyle->addProperty("draw:fill", "bitmap");
+        m_currentDrawStyle->addProperty("draw:fill-image-name", fillImageName);
         if (m_context->type != SlideMaster) {
             if (!m_recentSourceName.isEmpty()) {
                 const QSize size(imageSize(m_recentSourceName));
                 kDebug() << "SIZE:" << size;
                 if (size.isValid()) {
-                    m_currentDrawStyle.addProperty("draw:fill-image-width",
+                    m_currentDrawStyle->addProperty("draw:fill-image-width",
                         MSOOXML::Utils::cmString(POINT_TO_CM(size.width())));
-                    m_currentDrawStyle.addProperty("draw:fill-image-height",
+                    m_currentDrawStyle->addProperty("draw:fill-image-height",
                         MSOOXML::Utils::cmString(POINT_TO_CM(size.height())));
                 }
             }
@@ -881,6 +1032,7 @@ KoFilter::ConversionStatus PptxXmlSlideReader::read_spTree()
     d->presentationStyleNameCount = 0;
 #endif
     d->shapeNumber = 0;
+
     QByteArray placeholderEl;
     QBuffer placeholderElBuffer(&placeholderEl);
     placeholderElBuffer.open(QIODevice::WriteOnly);
@@ -934,8 +1086,8 @@ KoFilter::ConversionStatus PptxXmlSlideReader::read_ph()
     const QXmlStreamAttributes attrs(attributes());
     // Specifies the placeholder index. This is used when applying templates or changing
     // layouts to match a placeholder on one template/master to another.
-    TRY_READ_ATTR_WITHOUT_NS(idx)
-    kDebug() << "idx:" << idx;
+    TRY_READ_ATTR_WITHOUT_NS_INTO(idx, d->phIdx)
+    kDebug() << "idx:" << d->phIdx;
 
     // Specifies the size of a placeholder.
     // The possible values for this attribute are defined by the ST_PlaceholderSize simple type (§19.7.9), p.2987.
@@ -950,6 +1102,16 @@ KoFilter::ConversionStatus PptxXmlSlideReader::read_ph()
     // Mark this shape as a place holder.
     m_isPlaceHolder = true;
 
+    const QString styleId(d->phStyleId());
+    kDebug() << "styleId:" << styleId;
+    if (m_context->type == Slide) {
+        if (m_context->slideLayoutProperties->styles.contains(styleId) && m_context->slideLayoutProperties->styles[styleId].contains(m_currentListLevel)) {
+            m_currentParagraphStyle = m_context->slideLayoutProperties->styles[styleId][m_currentListLevel];
+            m_currentParagraphStylePredefined = true;
+        }
+    }
+    else if (m_context->type == SlideLayout) {
+    }
     readNext();
     READ_EPILOGUE
 }
@@ -962,13 +1124,14 @@ KoFilter::ConversionStatus PptxXmlSlideReader::read_ph()
  Parent elements:
  - [done] sp (§19.3.1.43)
  Child elements:
- - bodyPr (Body Properties) §21.1.2.1.1
+ - [done] bodyPr (Body Properties) §21.1.2.1.1
  - [done] lstStyle (Text List Styles) §21.1.2.4.12
  - [done] p (Text Paragraphs) §21.1.2.2.6
 
  Only used for Slide type.
 */
 //! @todo support all child elements
+//! CASE #P526
 KoFilter::ConversionStatus PptxXmlSlideReader::read_txBody()
 {
     READ_PROLOGUE
@@ -984,8 +1147,9 @@ KoFilter::ConversionStatus PptxXmlSlideReader::read_txBody()
     body->startElement("draw:text-box"); // CASE #P436
 
     m_lstStyleFound = false;
+    m_prevListLevel = 0;
+    m_currentListLevel = 0;
     m_pPr_lvl = 0;
-    const bool isOutline = MSOOXML::Utils::ST_PlaceholderType_to_ODF(d->phType) == QLatin1String("outline");
 
     MSOOXML::Utils::XmlWriteBuffer listBuf;
     body = listBuf.setWriter(body);
@@ -994,25 +1158,23 @@ KoFilter::ConversionStatus PptxXmlSlideReader::read_txBody()
         readNext();
         kDebug() << *this;
         if (isStartElement()) {
-            TRY_READ_IF_NS(a, lstStyle)
+            TRY_READ_IF_NS(a, bodyPr)
+            ELSE_TRY_READ_IF_NS(a, lstStyle)
             else if (qualifiedName() == QLatin1String("a:p")) {
-                // Buffer each text:p, because we have to write after
-                // the child elements are generated
-                MSOOXML::Utils::XmlWriteBuffer paragraphBuf;
-                if (isOutline) {
-                    body = paragraphBuf.setWriter(body);
-                }
                 TRY_READ(DrawingML_p);
-                if (isOutline) { // CASE #P612
-                     body = paragraphBuf.originalWriter();
-                    (void)paragraphBuf.releaseWriter();
-                }
             }
 //! @todo add ELSE_WRONG_FORMAT
         }
         BREAK_IF_END_OF(CURRENT_EL);
     }
 
+    if (m_prevListLevel > 0) {
+        for(; m_prevListLevel > 0; --m_prevListLevel) {
+            body->endElement(); // text:list-item
+            body->endElement(); // text:list
+        }
+    }
+
     if (m_lstStyleFound) {
         body = listBuf.originalWriter();
         body->startElement("text:list");
@@ -1033,7 +1195,7 @@ KoFilter::ConversionStatus PptxXmlSlideReader::read_txBody()
 
 #undef CURRENT_EL
 #define CURRENT_EL graphicFrame
-//! graphicFrame 
+//! graphicFrame
 /*!
   This element specifies the existence of a graphics frame. This frame contains a graphic that was generated
   by an external source and needs a container in which to be displayed on the slide surface.
@@ -1043,17 +1205,22 @@ KoFilter::ConversionStatus PptxXmlSlideReader::read_txBody()
   Child Elements:
     - extLst (Extension List with Modification Flag) (§4.2.4)
     - graphic (Graphic Object) (§5.1.2.1.16)
-    - nvGraphicFramePr (Non-Visual Properties for a Graphic Frame) (§4.4.1.27)
+    - [done] nvGraphicFramePr (Non-Visual Properties for a Graphic Frame) (§4.4.1.27)
     - xfrm (2D Transform for Graphic Frame)
 */
 KoFilter::ConversionStatus PptxXmlSlideReader::read_graphicFrame()
 {
     READ_PROLOGUE
     m_svgX = m_svgY = m_svgWidth = m_svgHeight = 0;
+
+    MSOOXML::Utils::XmlWriteBuffer buffer;
+    body = buffer.setWriter(body);
+
     while (!atEnd()) {
         readNext();
         if (isStartElement()) {
             TRY_READ_IF_NS(a, graphic)
+            ELSE_TRY_READ_IF(nvGraphicFramePr)
             else if (qualifiedName() == "p:xfrm") {
                 read_xfrm_p();
             }
@@ -1061,6 +1228,49 @@ KoFilter::ConversionStatus PptxXmlSlideReader::read_graphicFrame()
         }
         BREAK_IF_END_OF(CURRENT_EL);
     }
+
+    body = buffer.originalWriter();
+    body->startElement("draw:frame");
+    body->addAttribute("draw:name", m_cNvPrName);
+    body->addAttribute("draw:layer", "layout");
+    body->addAttribute("svg:x", EMU_TO_CM_STRING(m_svgX));
+    body->addAttribute("svg:y", EMU_TO_CM_STRING(m_svgY));
+    body->addAttribute("svg:width", EMU_TO_CM_STRING(m_svgWidth));
+    body->addAttribute("svg:height", EMU_TO_CM_STRING(m_svgHeight));
+
+    (void)buffer.releaseWriter();
+
+    body->endElement();
+
+    READ_EPILOGUE
+}
+
+#undef CURRENT_EL
+#define CURRENT_EL nvGraphicFramePr
+//! graphicFramePr handler
+/*
+ Parent elements:
+ - [done] graphicFrame (§19.3.1.21)
+
+ Child elements:
+ - cNvGraphicFramePr (Non-Visual Graphic Frame Drawing Properties) §19.3.1.9
+ - cNvPr (Non-Visual Drawing Properties) §19.3.1.12
+ - [done] nvPr (Non-Visual Properties) §19.3.1.33
+
+*/
+KoFilter::ConversionStatus PptxXmlSlideReader::read_nvGraphicFramePr()
+{
+    READ_PROLOGUE
+    while (!atEnd()) {
+        readNext();
+        if (isStartElement()) {
+            if (qualifiedName() == "p:cNvPr") {
+                read_cNvPr_p();
+            }
+            ELSE_TRY_READ_IF(nvGraphicFramePr) // This should be fixed later
+        }
+        BREAK_IF_END_OF(CURRENT_EL);
+    }
     READ_EPILOGUE
 }
 
@@ -1087,10 +1297,11 @@ KoFilter::ConversionStatus PptxXmlSlideReader::read_graphicFrame()
     - wavAudioFile (Audio from WAV File) §20.1.3.7
 */
 //! @todo support all child elements
-KoFilter::ConversionStatus MSOOXML_CURRENT_CLASS::read_nvPr()
+KoFilter::ConversionStatus PptxXmlSlideReader::read_nvPr()
 {
     READ_PROLOGUE
     d->phType.clear();
+    d->phIdx.clear();
     while (!atEnd()) {
         readNext();
         kDebug() << *this;
@@ -1106,6 +1317,26 @@ KoFilter::ConversionStatus MSOOXML_CURRENT_CLASS::read_nvPr()
 #undef MSOOXML_CURRENT_NS
 #define MSOOXML_CURRENT_NS "p"
 
+#undef CURRENT_EL
+#define CURRENT_EL cNvPr
+//! p:nvPr handler
+KoFilter::ConversionStatus PptxXmlSlideReader::read_cNvPr_p()
+{
+    READ_PROLOGUE
+
+    const QXmlStreamAttributes attrs(attributes());
+
+    READ_ATTR_WITHOUT_NS_INTO(id, m_cNvPrId)
+    TRY_READ_ATTR_WITHOUT_NS_INTO(name, m_cNvPrName)
+
+    while (!atEnd()) {
+        readNext();
+        BREAK_IF_END_OF(CURRENT_EL);
+    }
+
+    READ_EPILOGUE
+}
+
 #undef CURRENT_EL
 #define CURRENT_EL xfrm
 //! p:xfrm handler that redirects to a:xfrm
@@ -1145,3 +1376,4 @@ KoFilter::ConversionStatus PptxXmlSlideReader::read_xfrm_p()
 #define DRAWINGML_PIC_NS "p" // DrawingML/Picture
 
 #include <MsooXmlCommonReaderDrawingMLImpl.h> // this adds p:pic, etc.
+#include <MsooXmlDrawingReaderTableImpl.h> //this adds a:tbl
