@@ -43,6 +43,8 @@ class PptxShapeProperties
 {
 public:
     PptxShapeProperties();
+    PptxShapeProperties(const PptxShapeProperties &other);
+    PptxShapeProperties& operator=(const PptxShapeProperties &other);
 
     //! p:spPr/a:xfrm/a:off@x value in EMU
     int x;
@@ -66,8 +68,12 @@ public:
     ~PptxSlideProperties();
     //! Shapes ordered by position
     QList<PptxShapeProperties*> shapes;
-
+    //! Shapes map addressed by type
+    QMap<QString, PptxShapeProperties*> shapesMap;
+    //! Clear the shapes and shapesMap lists.
     void clear();
+private:
+    Q_DISABLE_COPY(PptxSlideProperties)
 };
 
 //! Properties of a single placeholder for shapes defined by layouts
@@ -75,6 +81,7 @@ class PptxPlaceholder
 {
 public:
     PptxPlaceholder();
+    PptxPlaceholder(const PptxShapeProperties &other);
     ~PptxPlaceholder();
     //! Writers placeholder's attributes (svg:x, etc.) to @a writer
     void writeAttributes(KoXmlWriter* writer);
@@ -90,17 +97,65 @@ class PptxSlideLayoutProperties
 public:
     PptxSlideLayoutProperties();
     ~PptxSlideLayoutProperties();
-    QMap<QString, PptxPlaceholder*> placeholders; //! placeholders objects are owned by this object
-    QString styleName;
+    //! Shapes ordered by position
+    QList<PptxShapeProperties*> shapes;
+    //! Shapes map addressed by type
+    QMap<QString, PptxShapeProperties*> shapesMap;
+    //! placeholders objects are owned by this object
+    QMap<QString, PptxPlaceholder*> placeholders;
+    //! The presentation:presentation-page-layout-name
+    QString pageLayoutStyleName;
+    //! Map of paragraph-styles with the styleId as outer-key and the listlevel as inner-key.
+    QMap<QString, QMap<int,KoGenStyle> > styles;
+private:
+    Q_DISABLE_COPY(PptxSlideLayoutProperties)
 };
 
-//! Data structure collecting information about master
+//! Data structure collecting information about single text style for one list level defined by master slide
+//! This can be bodyStyle, otherStyle, tileStyle
+//! Used by PptxSlideMasterTextStyle
+//! Attributes here are of type QVariant, so it is possible to test if these are present.
+class PptxSlideMasterListLevelTextStyle
+{
+public:
+    PptxSlideMasterListLevelTextStyle();
+    ~PptxSlideMasterListLevelTextStyle();
+    KoCharacterStyle* m_characterStyle;
+private:
+    Q_DISABLE_COPY(PptxSlideMasterListLevelTextStyle)
+};
+
+//! Data structure collecting information about single text style defined by master slide
+//! This can be bodyStyle, otherStyle, tileStyle
+class PptxSlideMasterTextStyle
+{
+public:
+    PptxSlideMasterTextStyle();
+    ~PptxSlideMasterTextStyle();
+    void clear();
+    //! @return text style for list level @a level
+    //! @par level can be 1..9, otherwise 0 is returned.
+    //! Returned object is owned by PptxSlideMasterTextStyle.
+    PptxSlideMasterListLevelTextStyle *listStyle(uint level);
+private:
+    Q_DISABLE_COPY(PptxSlideMasterTextStyle)
+    QVector<PptxSlideMasterListLevelTextStyle*> m_listStyles;
+};
+
+//! Data structure collecting information about master slide
 class PptxSlideMasterPageProperties
 {
 public:
     PptxSlideMasterPageProperties();
+    void clear();
     void addDrawingPageProperty(const QByteArray& property, const QByteArray& value);
     void saveDrawingPageProperties(KoGenStyle* style);
+
+    PptxSlideMasterTextStyle* textStyle(const QString& style);
+    PptxSlideMasterTextStyle titleStyle;
+    PptxSlideMasterTextStyle bodyStyle;
+    PptxSlideMasterTextStyle otherStyle;
+    
 private:
     QMap<QByteArray, QByteArray> m_drawingPageProperties;
 };
@@ -142,7 +197,10 @@ protected:
     KoFilter::ConversionStatus read_ph();
     KoFilter::ConversionStatus read_txBody();
     KoFilter::ConversionStatus read_graphicFrame();
+    KoFilter::ConversionStatus read_nvGraphicFramePr();
+
     KoFilter::ConversionStatus read_xfrm_p();
+    KoFilter::ConversionStatus read_cNvPr_p();
     KoFilter::ConversionStatus read_oleObj();
 
 //    KoGenStyle m_currentPageStyle;
@@ -151,6 +209,7 @@ protected:
 
     KoGenStyle m_currentMasterPageStyle; //!< set by read_sp()
     KoGenStyle m_currentPresentationPageLayoutStyle; //!< set by read_sp() for placeholders (KoGenStyle::PresentationPageLayoutStyle)
+    KoGenStyle m_currentPresentationStyle; //!< KoGenStyle::PresentationAutoStyle
 
     KoXmlWriter* m_placeholderElWriter;
 
@@ -161,6 +220,7 @@ private:
 
 #include <MsooXmlCommonReaderMethods.h>
 #include <MsooXmlCommonReaderDrawingMLMethods.h>
+#include <MsooXmlDrawingReaderTableMethods.h>
 };
 
 //! Context for PptxXmlSlideReader
@@ -176,7 +236,8 @@ public:
         PptxXmlSlideReader::Type _type, PptxSlideProperties* _slideProperties,
         PptxSlideLayoutProperties* _slideLayoutProperties,
         PptxSlideMasterPageProperties* _slideMasterPageProperties,
-        MSOOXML::MsooXmlRelationships& _relationships);
+        MSOOXML::MsooXmlRelationships& _relationships,
+        QMap<int, QString> _commentAuthors);
     PptxImport* import;
     const QString path;
     const QString file;
@@ -187,6 +248,7 @@ public:
     PptxSlideLayoutProperties* slideLayoutProperties;
     PptxSlideMasterPageProperties* slideMasterPageProperties;
     QString pageDrawStyleName; //!< written in read_sldInternal()
+    QMap<int, QString> commentAuthors;
 };
 
 #endif
