@@ -30,6 +30,15 @@
 #include "records.h"
 #include "objects.h"
 
+// libmso
+//#include <generated/simpleParser.h>
+//#include <generated/leinputstream.h>
+#include "generated/simpleParser.h"
+#include "generated/leinputstream.h"
+#include "drawstyle.h"
+#include "ODrawToOdf.h"
+#include "pictures.h"
+
 namespace Swinder
 {
 // rich-text, unicode, far-east support Excel string
@@ -54,9 +63,9 @@ public:
 
     void setRichText(bool r);
 
-    UString str() const;
+    QString str() const;
 
-    void setStr(const UString& str);
+    void setStr(const QString& str);
 
     std::map<unsigned, unsigned> formatRuns() const;
 
@@ -315,7 +324,7 @@ public:
 
     unsigned sheetCount() const;
 
-    UString bookName() const;
+    QString bookName() const;
 
     virtual void setData(unsigned size, const unsigned char* data, const unsigned int* continuePositions);
 
@@ -356,9 +365,9 @@ public:
 
     void setSheetIndex(unsigned sheetIndex);
 
-    UString externName() const;
+    QString externName() const;
 
-    void setExternName(const UString& name);
+    void setExternName(const QString& name);
 
     virtual void setData(unsigned size, const unsigned char* data, const unsigned int* continuePositions);
 
@@ -582,9 +591,9 @@ public:
 
     ~NameRecord();
 
-    UString definedName() const;
+    QString definedName() const;
 
-    void setDefinedName(const UString& name);
+    void setDefinedName(const QString& name);
 
     unsigned sheetIndex() const;
 
@@ -730,14 +739,14 @@ public:
      *
      * \sa setLabel
      */
-    UString label() const;
+    QString label() const;
 
     /**
      * Sets the label string.
      *
      * \sa label
      */
-    void setLabel(const UString& l);
+    void setLabel(const QString& l);
 
     virtual void setData(unsigned size, const unsigned char* data, const unsigned int* continuePositions);
 
@@ -794,15 +803,15 @@ public:
     /**
       Returns the string at specified index.
       Note that index must be less than count().
-      If index is not valid, this will return UString::null.
+      If index is not valid, this will return QString::null.
      */
-    UString stringAt(unsigned index) const;
+    QString stringAt(unsigned index) const;
 
     /**
       Returnsformat runs for the string at specified index.
       Format runs are a mapping from character-index -> font index.
      */
-    std::map<unsigned,unsigned> formatRunsAt(unsigned index) const;
+    std::map<unsigned, unsigned> formatRunsAt(unsigned index) const;
 
     virtual const char* name() const {
         return "SST";
@@ -839,7 +848,7 @@ public:
 class TxORecord : public Record
 {
 public:
-    UString m_text;
+    QString m_text;
     static const unsigned id;
     TxORecord(Workbook *book);
     virtual ~TxORecord();
@@ -856,6 +865,7 @@ public:
 class MsoDrawingRecord : public Record, public DrawingObject
 {
 public:
+    //MSO::OfficeArtDgContainer m_container;
     static const unsigned id;
     MsoDrawingRecord(Workbook *book) : Record(book) {}
     virtual ~MsoDrawingRecord() {}
@@ -872,16 +882,27 @@ public:
 class MsoDrawingBlibItem
 {
 public:
-    std::string id;
-    std::string filename;
-    explicit MsoDrawingBlibItem() {}
+    //enum Type { Picture, ... };
+    PictureReference m_picture;
+    explicit MsoDrawingBlibItem(const PictureReference &picture);
 };
 
-class MsoDrawingGroupRecord : public Record, private DrawingObject
+class Picture
 {
 public:
-    std::vector< MsoDrawingBlibItem* > m_items;
+    /// The unique identifier of the picture.
+    QString m_id;
+    /// The path and filename of the picture-file in the KoStore. So, can be something like "Pictures/TheUniqueIdentifierOfThePicture.jpg" for example.
+    QString m_filename;
+    /// The position of the picture in the sheet.
+    unsigned long m_colL, m_dxL, m_rwT, m_dyT, m_colR, m_dxR, m_rwB, m_dyB;
+    Picture(MsoDrawingRecord *record, const PictureReference &picture);
+};
 
+class MsoDrawingGroupRecord : public Record
+{
+public:
+    QList< MsoDrawingBlibItem* > m_items;
     static const unsigned id;
     MsoDrawingGroupRecord(Workbook *book);
     virtual ~MsoDrawingGroupRecord();
@@ -1395,26 +1416,6 @@ private:
     Private* d;
 };
 
-class Picture
-{
-public:
-    std::string m_id;
-    std::string m_filename;
-    unsigned long m_colL, m_dxL, m_rwT, m_dyT, m_colR, m_dxR, m_rwB, m_dyB;
-    Picture(MsoDrawingRecord *record, MsoDrawingBlibItem *item) {
-        m_id = item->id;
-        m_filename = item->filename;
-        m_colL = record->m_colL;
-        m_dxL = record->m_dxL;
-        m_rwT = record->m_rwT;
-        m_dyT = record->m_dyT;
-        m_colR = record->m_colR;
-        m_dxR = record->m_dxR;
-        m_rwB = record->m_rwB;
-        m_dyB = record->m_dyB;
-    }
-};
-
 class ExcelReader
 {
 public:
@@ -1455,13 +1456,13 @@ public:
         NativeFormat = 0x000e
     };
 
-    static UString formatToString(Format format);
+    static QString formatToString(Format format);
 
     Format format() const;
     void setFormat( Format format );
 
-    UString imagePath() const;
-    void setImagePath( UString imagePath );
+    QString imagePath() const;
+    void setImagePath( QString imagePath );
 
     unsigned imageSize() const;
     void setImageSize( unsigned imageSize );
