@@ -22,7 +22,6 @@
 
 #include <Charting.h>
 
-#include "ustring.h"
 #include "formulas.h"
 #include "records.h"
 #include "format.h"
@@ -35,8 +34,9 @@
 #include <iostream>
 #include <vector>
 
-#include <QRect>
-#include <QList>
+#include <QtCore/QList>
+#include <QtCore/QRect>
+#include <QtCore/QString>
 
 namespace Swinder
 {
@@ -46,22 +46,7 @@ class DrawingObject
 public:
     enum Property {
         pid = 0x0104, // identifier for pictures
-        itxid = 0x0080, // identifier for text
-        cxk = 0x0158, // where on the shape the connection points are
-        fillColor = 0x0181, // foreground color of the fill
-        fillBackColor = 0x0183, // background color of the fill
-        fillCrMod = 0x0185, // background color of the fill for black-white displays
-        fillStyleBooleanProperties = 0x01bf, // 32-bit field of Bollean properties for the fill style
-        lineColor = 0x01c0, // foreground color of the line
-        lineCrMod = 0x01c3, // foreground color of the line for black-white displays
-        shadowColor = 0x0201, // primary color of the shadow
-        shadowCrMod = 0x0203, // primary color of the shadow for black-white displays
-        shadowStyleBooleanProperties = 0x023f, // 32-bit field of Bollean properties for shadows
-        groupShapeBooleanProperties = 0x03bf, // 32-bit field of Bollean properties for a shape or a group
-        lineStyleBooleanProperties = 0x01ff, // 32-bit field of Bollean properties for the line style
-        pictureContrast = 0x7f,
-        textBooleanProperties = 0x00bf,
-        wzName = 0x0380
+        itxid = 0x0080 // identifier for text
     };
 
     static const char* propertyName(Property p);
@@ -147,7 +132,7 @@ class PictureObject : public Object
 public:
     PictureObject(unsigned long id) : Object(Picture, id), m_offset(0), m_size(0) {}
     virtual ~PictureObject() {}
-    
+
     /// Enumeration of possible image types.
     enum Type { EnhancedMetafile, Bitmap, Unspecified };
     /// Returns the type of the image.
@@ -158,7 +143,7 @@ public:
     void setType(const Type &t) {
         m_type = t;
     }
-    
+
     /// Returns the offset of the picture in the control stream.
     uint controlStreamOffset() {
         return m_offset;
@@ -172,7 +157,7 @@ public:
         m_offset = offset;
         m_size = size;
     }
-    
+
     /**
      * Returns the filename of the embedded storage. This can be empty if the
      * picture is not located in an embedded but e.g. in a control stream.
@@ -190,7 +175,7 @@ private:
     Type m_type;
     uint m_offset, m_size;
     std::string m_storage;
-    
+
 };
 
 /**
@@ -202,15 +187,15 @@ public:
     NoteObject(unsigned long id) : Object(Note, id) {}
     virtual ~NoteObject() {}
     /// Returns the comment.
-    UString note() const {
+    QString note() const {
         return m_note;
     }
     /// Set the comment.
-    void setNote(const UString &n) {
+    void setNote(const QString &n) {
         m_note = n;
     }
 private:
-    UString m_note;
+    QString m_note;
 };
 
 class ChartObject : public Object
