@@ -21,9 +21,10 @@
 #include "cell.h"
 #include "sheet.h"
 #include "workbook.h"
-#include "ustring.h"
 #include "utils.h"
 
+#include <PointStorage.h>
+
 #include <iostream>
 #include <map>
 #include <QPoint>
@@ -40,7 +41,7 @@ class Sheet::Private
 {
 public:
     Workbook* workbook;
-    UString name;
+    QString name;
 
     // hash to store cell, FIXME replace with quad-tree
     QHash<unsigned, Cell*> cells;
@@ -49,17 +50,20 @@ public:
     QHash<unsigned, unsigned> maxCellsInRow;
     QHash<unsigned, Column*> columns;
     QHash<unsigned, Row*> rows;
+    KSpread::PointStorage<Hyperlink> hyperlinks;
+    KSpread::PointStorage<QList<Picture*> > pictures;
+    KSpread::PointStorage<QList<ChartObject*> > charts;
 
     bool visible;
     bool protect;
     bool autoCalc;
 
-    UString leftHeader;
-    UString centerHeader;
-    UString rightHeader;
-    UString leftFooter;
-    UString centerFooter;
-    UString rightFooter;
+    QString leftHeader;
+    QString centerHeader;
+    QString rightHeader;
+    QString leftFooter;
+    QString centerFooter;
+    QString rightFooter;
 
     double leftMargin;
     double rightMargin;
@@ -76,7 +80,7 @@ public:
     bool isPageBreakViewEnabled;
 
     unsigned long passwd;
-    UString backgroundImagePath;
+    QString backgroundImagePath;
 
     QList<VerticalPageBreak> verticalPageBreaks;
     QList<HorizontalPageBreak> horizontalPageBreaks;
@@ -116,6 +120,13 @@ void Sheet::setAutoCalc(bool a)
 
 void Sheet::clear()
 {
+    // delete all cell data
+    for (int i = 0; i < d->pictures.count(); i++) {
+        qDeleteAll(d->pictures.data(i));
+    }
+    for (int i = 0; i < d->charts.count(); i++) {
+        qDeleteAll(d->charts.data(i));
+    }
     // delete all cells
     qDeleteAll(d->cells);
     d->cells.clear();
@@ -147,12 +158,12 @@ void Sheet::clear()
     d->passwd = 0; // password protection disabled
 }
 
-UString Sheet::name() const
+QString Sheet::name() const
 {
     return d->name;
 }
 
-void Sheet::setName(const UString& name)
+void Sheet::setName(const QString& name)
 {
     d->name = name;
 }
@@ -258,62 +269,62 @@ void Sheet::setProtect(bool p)
     d->protect = p;
 }
 
-UString Sheet::leftHeader() const
+QString Sheet::leftHeader() const
 {
     return d->leftHeader;
 }
 
-void Sheet::setLeftHeader(const UString& h)
+void Sheet::setLeftHeader(const QString& h)
 {
     d->leftHeader = h;
 }
 
-UString Sheet::centerHeader() const
+QString Sheet::centerHeader() const
 {
     return d->centerHeader;
 }
 
-void Sheet::setCenterHeader(const UString& h)
+void Sheet::setCenterHeader(const QString& h)
 {
     d->centerHeader = h;
 }
 
-UString Sheet::rightHeader() const
+QString Sheet::rightHeader() const
 {
     return d->rightHeader;
 }
 
-void Sheet::setRightHeader(const UString& h)
+void Sheet::setRightHeader(const QString& h)
 {
     d->rightHeader = h;
 }
 
-UString Sheet::leftFooter() const
+QString Sheet::leftFooter() const
 {
     return d->leftFooter;
 }
 
-void Sheet::setLeftFooter(const UString& h)
+void Sheet::setLeftFooter(const QString& h)
 {
     d->leftFooter = h;
 }
 
-UString Sheet::centerFooter() const
+QString Sheet::centerFooter() const
 {
     return d->centerFooter;
 }
 
-void Sheet::setCenterFooter(const UString& h)
+void Sheet::setCenterFooter(const QString& h)
 {
     d->centerFooter = h;
 }
 
-UString Sheet::rightFooter() const
+QString Sheet::rightFooter() const
 {
     return d->rightFooter;
 }
 
-void Sheet::setRightFooter(const UString& h)
+void Sheet::setRightFooter(const QString& h)
 {
     d->rightFooter = h;
 }
@@ -438,12 +449,12 @@ void Sheet::setPassword(unsigned long hash)
     d->passwd = hash;
 }
 
-void Sheet::setBackgroundImage( const UString& imagePath )
+void Sheet::setBackgroundImage( const QString& imagePath )
 {
     d->backgroundImagePath = imagePath;
 }
 
-UString Sheet::backgroundImage()
+QString Sheet::backgroundImage()
 {
     return d->backgroundImagePath;
 }
@@ -468,6 +479,80 @@ QList<HorizontalPageBreak> Sheet::horizontalPageBreaks()
     return d->horizontalPageBreaks;
 }
 
+Hyperlink Sheet::hyperlink(unsigned column, unsigned row) const
+{
+    return d->hyperlinks.lookup(column+1, row+1);
+}
+
+void Sheet::setHyperlink(unsigned column, unsigned row, const Hyperlink& link)
+{
+    if (link.isValid)
+        d->hyperlinks.insert(column+1, row+1, link);
+    else
+        d->hyperlinks.take(column+1, row+1);
+}
+
+QList<Picture*> Sheet::pictures(unsigned column, unsigned row) const
+{
+    return d->pictures.lookup(column+1, row+1);
+}
+
+void Sheet::setPictures(unsigned column, unsigned row, const QList<Picture*>& pictures)
+{
+    if (pictures.isEmpty())
+        d->pictures.take(column+1, row+1);
+    else
+        d->pictures.insert(column+1, row+1, pictures);
+}
+
+void Sheet::addPicture(unsigned column, unsigned row, Picture* picture)
+{
+    QList<Picture*> pics = pictures(column, row);
+    pics.append(picture);
+    setPictures(column, row, pics);
+}
+
+QList<ChartObject*> Sheet::charts(unsigned column, unsigned row) const
+{
+    return d->charts.lookup(column+1, row+1);
+}
+
+void Sheet::setCharts(unsigned column, unsigned row, const QList<ChartObject*>& charts)
+{
+    if (charts.isEmpty())
+        d->charts.take(column+1, row+1);
+    else
+        d->charts.insert(column+1, row+1, charts);
+}
+
+void Sheet::addChart(unsigned column, unsigned row, ChartObject* chart)
+{
+    QList<ChartObject*> chrts = charts(column, row);
+    chrts.append(chart);
+    setCharts(column, row, chrts);
+}
+
+#ifdef SWINDER_XLS2RAW
+void Sheet::dumpStats()
+{
+    int ndValue = 0, ndFormula = 0, ndFormat = 0, ndColumnSpan = 0, ndRowSpan = 0, ndCovered = 0, ndColumnRepeat = 0, ndHyperlink = 0, ndNote = 0, ndPictures = 0, ndCharts = 0;
+    foreach (Cell* c, d->cells) {
+        if (c->value() != Value()) ndValue++;
+        if (!c->formula().isEmpty()) ndFormula++;
+        if (c->format() != Format()) ndFormat++;
+        if (c->columnSpan() != 1) ndColumnSpan++;
+        if (c->rowSpan() != 1) ndRowSpan++;
+        if (c->isCovered()) ndCovered++;
+        if (c->columnRepeat() != 1) ndColumnRepeat++;
+        if (c->hasHyperlink()) ndHyperlink++;
+        if (!c->note().isEmpty()) ndNote++;
+        if (c->pictures().size()) ndPictures++;
+        if (c->charts().size()) ndCharts++;
+    }
+    printf("    rows: %d\n  cols: %d\n  cells: %d\n", d->rows.size(), d->columns.size(), d->cells.size());
+    printf("       values: %d\n       formulas: %d\n       formats: %d\n       colspans: %d\n       rowspans: %d\n       covered: %d\n       colrepeat: %d\n       hyperlink: %d\n       note: %d\n       pics: %d\n       charts: %d\n", ndValue, ndFormula, ndFormat, ndColumnSpan, ndRowSpan, ndCovered, ndColumnRepeat, ndHyperlink, ndNote, ndPictures, ndCharts);
+}
+#endif
 
 class Column::Private
 {
@@ -475,7 +560,7 @@ public:
     Sheet* sheet;
     unsigned index;
     double width;
-    Format format;
+    const Format* format;
     bool visible;
     unsigned outlineLevel;
     bool collapsed;
@@ -490,6 +575,7 @@ Column::Column(Sheet* sheet, unsigned index)
     d->visible = true;
     d->outlineLevel = 0;
     d->collapsed = false;
+    d->format  = 0;
 }
 
 Column::~Column()
@@ -522,10 +608,12 @@ void Column::setWidth(double w)
 
 const Format& Column::format() const
 {
-    return d->format;
+    static const Format null;
+    if (!d->format) return null;
+    return *(d->format);
 }
 
-void Column::setFormat(const Format& f)
+void Column::setFormat(const Format* f)
 {
     d->format = f;
 }
@@ -598,7 +686,7 @@ public:
     Sheet* sheet;
     unsigned index;
     double height;
-    Format format;
+    const Format* format;
     bool visible;
     unsigned outlineLevel;
     bool collapsed;
@@ -613,6 +701,7 @@ Row::Row(Sheet* sheet, unsigned index)
     d->visible = true;
     d->outlineLevel = 0;
     d->collapsed = false;
+    d->format  = 0;
 }
 
 Row::~Row()
@@ -642,10 +731,12 @@ void Row::setHeight(double w)
 
 const Format& Row::format() const
 {
-    return d->format;
+    static const Format null;
+    if (!d->format) return null;
+    return *(d->format);
 }
 
-void Row::setFormat(const Format& f)
+void Row::setFormat(const Format* f)
 {
     d->format = f;
 }
