@@ -50,6 +50,7 @@
 
 #include <QBrush>
 #include <QRegExp>
+#include <QString>
 
 #include "NumberFormatParser.h"
 
@@ -82,7 +83,8 @@ XlsxXmlWorksheetReaderContext::XlsxXmlWorksheetReaderContext(
     const XlsxComments& _comments,
     const XlsxStyles& _styles,
     MSOOXML::MsooXmlRelationships& _relationships,
-    XlsxImport* _import
+    XlsxImport* _import,
+    int& numberOfOleObjects
 )
         : MSOOXML::MsooXmlReaderContext(&_relationships)
         , worksheetNumber(_worksheetNumber)
@@ -95,6 +97,7 @@ XlsxXmlWorksheetReaderContext::XlsxXmlWorksheetReaderContext(
         , import(_import)
         , path(_path)
         , file(_file)
+        , numberOfOleObjects(numberOfOleObjects)
 {
 }
 
@@ -124,6 +127,9 @@ public:
     QString hyperlink;
     QList<XlsxXmlDrawingReaderContext*> drawings;
 
+    //QPair< oleObjectFile, imageReplacementFile>
+    QList< QPair<QString,QString> > oleObjects;
+
     Cell(Sheet* s, int columnIndex, int rowIndex) : sheet(s), column(columnIndex), row(rowIndex), rowsMerged(1), columnsMerged(1) {}
     ~Cell() { qDeleteAll(drawings); }
 };
@@ -224,7 +230,8 @@ public:
      : q( qq ),
        warningAboutWorksheetSizeDisplayed(false),
        drawingNumber(0),
-       sheet(new Sheet)
+       sheet(new Sheet),
+       numberOfOleObjects(0)
     {
     }
     ~Private()
@@ -238,6 +245,7 @@ public:
     int drawingNumber;
     Sheet* sheet;
     QHash<int, Cell*> sharedFormulas;
+    int numberOfOleObjects;
 };
 
 XlsxXmlWorksheetReader::XlsxXmlWorksheetReader(KoOdfWriters *writers)
@@ -269,8 +277,7 @@ KoFilter::ConversionStatus XlsxXmlWorksheetReader::read(MSOOXML::MsooXmlReaderCo
     Q_ASSERT(m_context);
     const KoFilter::ConversionStatus result = readInternal();
     m_context = 0;
-    if (result == KoFilter::OK)
-        return KoFilter::OK;
+
     return result;
 }
 
@@ -429,6 +436,7 @@ KoFilter::ConversionStatus XlsxXmlWorksheetReader::read_worksheet()
             ELSE_TRY_READ_IF(drawing)
             ELSE_TRY_READ_IF(hyperlinks)
             ELSE_TRY_READ_IF(picture)
+            ELSE_TRY_READ_IF(oleObjects)
         }
         BREAK_IF_END_OF(CURRENT_EL);
     }
@@ -548,6 +556,34 @@ KoFilter::ConversionStatus XlsxXmlWorksheetReader::read_worksheet()
                             //chart->m_chartExport->saveContent(m_context->import->outputStore(), manifest);
                         }
                     }
+
+                    QPair<QString,QString> oleObject;
+                    foreach( oleObject, cell->oleObjects ) {
+                        const QString olePath = oleObject.first;
+                        const QString previewPath = oleObject.second;
+                        body->startElement("draw:frame");
+                        //TODO find out the proper values
+                        body->addAttribute("svg:x", "1cm");
+                        body->addAttribute("svg:y", "1cm");
+                        body->addAttribute("svg:width", "5cm");
+                        body->addAttribute("svg:height", "5cm");
+
+                        body->startElement("draw:object-ole");
+                        body->addAttribute("xlink:href", olePath);
+                        body->addAttribute("xlink:type", "simple");
+                        body->addAttribute("xlink:show", "embed");
+                        body->addAttribute("xlink:actuate", "onLoad");
+                        body->endElement(); // draw:object-ole
+
+                        body->startElement("draw:image");
+                        body->addAttribute("xlink:href", previewPath);
+                        body->addAttribute("xlink:type", "simple");
+                        body->addAttribute("xlink:show", "embed");
+                        body->addAttribute("xlink:actuate", "onLoad");
+                        body->endElement(); // draw:image
+
+                        body->endElement(); // draw:frame
+                    }
                 }
                 body->endElement(); // table:table-cell
             }
@@ -982,8 +1018,8 @@ KoFilter::ConversionStatus XlsxXmlWorksheetReader::read_c()
         KoCharacterStyle cellCharacterStyle;
         cellFormat->setupCharacterStyle(m_context->styles, &cellCharacterStyle);
 
-        if( cellCharacterStyle.verticalAlignment() == QTextCharFormat::AlignSuperScript ||
-                cellCharacterStyle.verticalAlignment() == QTextCharFormat::AlignSubScript ) {
+        if( cellCharacterStyle.verticalAlignment() == QTextCharFormat::AlignSuperScript 
+            || cellCharacterStyle.verticalAlignment() == QTextCharFormat::AlignSubScript ) {
             KoGenStyle charStyle( KoGenStyle::TextStyle, "text" );
             cellCharacterStyle.saveOdf( charStyle );
             charStyleName = mainStyles->insert( charStyle, "T" );
@@ -994,14 +1030,14 @@ KoFilter::ConversionStatus XlsxXmlWorksheetReader::read_c()
         }
 
         /* depending on type: 18.18.11 ST_CellType (Cell Type), p. 2679:
-            b (Boolean) Cell containing a boolean.
-            d (Date) Cell contains a date in the ISO 8601 format.
-            e (Error) Cell containing an error.
-            inlineStr (Inline String) Cell containing an (inline) rich string, i.e., one not in
-            the shared string table. If this cell type is used, then
-            the cell value is in the is element rather than the v
-            element in the cell (c element).
-            n (Number) Cell containing a number.
+            b (Boolean)  Cell containing a boolean.
+            d (Date)     Cell contains a date in the ISO 8601 format.
+            e (Error)    Cell containing an error.
+            inlineStr    (Inline String) Cell containing an (inline) rich string, i.e. 
+                         one not in the shared string table. If this cell type is used,
+                         then the cell value is in the is element rather than the v
+                         element in the cell (c element).
+            n (Number)   Cell containing a number.
             s (Shared String) Cell containing a shared string.
             str (String) Cell containing a formula string.
 
@@ -1427,3 +1463,62 @@ KoFilter::ConversionStatus XlsxXmlWorksheetReader::read_picture()
     READ_EPILOGUE
 }
 
+#undef CURRENT_EL
+#define CURRENT_EL oleObjects
+
+KoFilter::ConversionStatus XlsxXmlWorksheetReader::read_oleObjects()
+{
+    READ_PROLOGUE
+    while (!atEnd()) {
+        readNext();
+        if( isStartElement() ) {
+            TRY_READ_IF(oleObject)
+        }
+        BREAK_IF_END_OF(CURRENT_EL);
+    }
+    READ_EPILOGUE
+}
+
+#undef CURRENT_EL
+#define CURRENT_EL oleObject
+
+KoFilter::ConversionStatus XlsxXmlWorksheetReader::read_oleObject()
+{
+    READ_PROLOGUE
+
+    const QXmlStreamAttributes attrs(attributes());
+    READ_ATTR_WITH_NS(r, id)
+    READ_ATTR_WITHOUT_NS(progId);
+
+    //For now we just copy the preview emf file and the embeded document,
+    //later we might convert it into ODF too.
+    //Preview files exist only for some files, please add the program generator
+    //for the files we will try to copy the preview image from
+    if(progId != "PowerPoint.Slide.12")
+        return KoFilter::OK;
+
+    ++(m_context->numberOfOleObjects);
+
+    const QString link = m_context->relationships->target(m_context->path, m_context->file, r_id);
+    QString fileName = link.right( link.lastIndexOf('/') +1 );
+    RETURN_IF_ERROR( copyFile(link, "", fileName) )
+
+    //In the OOXML specification the preview image would be represented as a relationship
+    //it is not specified how exactly that relationship is to be picked.
+    //However, to make things worse, the relationship seems that is not saved by MS2007,
+    //so, we must assume that the images in the media folder are ordered by appearance in the document
+    QString previewFileName = QString("image%1.emf").arg(m_context->numberOfOleObjects);
+    QString originalPreviewFilePath = "xl/media/" + previewFileName;
+    RETURN_IF_ERROR( copyFile(originalPreviewFilePath, "Pictures/", previewFileName) )
+
+    //TODO find out which cell to pick
+    Cell* cell = d->sheet->cell(0, 0, true);
+
+    cell->oleObjects << qMakePair<QString,QString>(fileName, previewFileName);
+
+    while (!atEnd()) {
+        readNext();
+        BREAK_IF_END_OF(CURRENT_EL);
+    }
+    READ_EPILOGUE
+}
