@@ -23,41 +23,40 @@
 #define __KWORD_LISTTABLE_H__
 
 #include "table.h"
-//Added by qt3to4:
-#include <Q3PtrList>
+#include <QList>
 
 /**
- * This class hold a list of tables. It just a wrapper for \verbatim QPtrList<Table> \endverbatim
+ * This class hold a list of tables. It just a wrapper for \verbatim QList<Table*> \endverbatim
  * to add a frame in a table if the table exists else to create a new table.
  */
-class ListTable: public Q3PtrList<Table>
+class ListTable: public QList<Table*>
 {
-	public:
-		/**
-		 * Constructors
-		 *
-		 * Creates a new instance of ListTable.
-		 */
-		ListTable();
+public:
+    /**
+     * Constructors
+     *
+     * Creates a new instance of ListTable.
+     */
+    ListTable();
 
-		/* 
-		 * Destructor
-		 */
-		virtual ~ListTable();
+    /*
+     * Destructor
+     */
+    virtual ~ListTable();
 
-		/**
-		 * Accessors
-		 */
+    /**
+     * Accessors
+     */
 
-		Table* isNewTable(QString);
+    Table* isNewTable(QString);
 
-		/**
-		 * Modifiers
-		 */
+    /**
+     * Modifiers
+     */
 
-		void add(Element*);
+    void add(Element*);
 
-	private:
+private:
 };
 
 #endif /* __KWORD_LISTTABLE_H__ */
