@@ -90,6 +90,7 @@ public:
 
     virtual void headerStart(wvWare::HeaderData::Type type);
     virtual void headerEnd();
+    virtual void headersMask(QList<bool> mask);
 
     virtual void footnoteStart();
     virtual void footnoteEnd();
@@ -118,9 +119,16 @@ public:
     QString masterPageName(void) const { return m_masterPageName_list.first(); }
     void set_writeMasterPageName(bool val) { m_writeMasterPageName = val; }
     bool writeMasterPageName(void) const { return m_writeMasterPageName; }
+    bool omittMasterPage(void) const { return m_omittMasterPage; }
     bool writingHeader(void) const { return m_writingHeader; }
     KoXmlWriter* headerWriter(void) const { return m_headerWriter; }
 
+    /**
+     * Return the item from m_headersMask corresponding with the actually
+     * processed section.  Return false if the Header document doesn't exist.
+     */
+    bool headersChanged(void) const;
+
     POLE::Storage* storage(void) const { return m_storage; }
     LEInputStream* data_stream(void) const { return m_data_stream; }
     LEInputStream* table_stream(void) const { return m_table_stream; }
@@ -194,12 +202,17 @@ private:
     bool m_firstOpen; //processing a first page header/footer
     QBuffer* m_buffer; //for odd and first page header/footer tags
     QBuffer* m_bufferEven; //for even header/footer tags
+
+    QList<KoGenStyle*> m_masterPageStyle_list; //master-page styles
+    QList<KoGenStyle*> m_pageLayoutStyle_list; //page-layout styles
+    QStringList m_masterPageName_list; //master-page names
+
+    QList<bool> m_headersMask; //mask informing of section's empty/nonempty header/footer stories
     QList<bool> m_hasHeader_list; //does master-page/page-layout require a header element
     QList<bool> m_hasFooter_list; //does master-page/page-layout require a footer element
 
-    QStringList m_masterPageName_list; //master-page names
-    QStringList m_pageLayoutName_list; //page-layout names
     bool m_writeMasterPageName; //whether to write the master-page name into a paragraph/table
+    bool m_omittMasterPage; //whether master-page style for current section has been omitted
 
     int m_initialFootnoteNumber;
     int m_initialEndnoteNumber;
@@ -211,8 +224,6 @@ private:
     LEInputStream* m_table_stream;
     LEInputStream* m_wdocument_stream;
     POLE::Storage* m_storage; // pointer to the pole storage
-
-    QMap<QByteArray, QString> m_picNames; //picture names shared in graphicshandler.cpp
 };
 
 #endif // DOCUMENT_H
