@@ -66,11 +66,12 @@ Parser9x::Position::Position( U32 cp, const PLCF<Word97::PCD>* plcfpcd ) :
 
 
 Parser9x::Parser9x( OLEStorage* storage, OLEStreamReader* wordDocument, const Word97::FIB& fib ) :
-        Parser( storage, wordDocument ), m_fib( fib ), m_table( 0 ), m_data( 0 ), m_properties( 0 ),
-        m_headers( 0 ), m_lists( 0 ), m_textconverter( 0 ), m_fields( 0 ), m_footnotes( 0 ), m_annotations( 0 ),
-        m_fonts( 0 ), m_drawings( 0 ), m_bookmark(0), m_plcfpcd( 0 ), m_tableRowStart( 0 ), m_tableRowLength( 0 ),
-        m_cellMarkFound( false ), m_remainingCells( 0 ), m_currentParagraph( new Paragraph ),
-        m_remainingChars( 0 ), m_sectionNumber( 0 ), m_subDocument( None ), m_parsingMode( Default )
+        Parser( storage, wordDocument ), m_fib( fib ), m_table( 0 ), m_data( 0 ), 
+        m_properties( 0 ), m_headers( 0 ), m_lists( 0 ), m_textconverter( 0 ), m_fields( 0 ),
+        m_footnotes( 0 ), m_annotations( 0 ), m_fonts( 0 ), m_drawings( 0 ), m_bookmarks(0),
+        m_plcfpcd( 0 ), m_tableRowStart( 0 ), m_tableRowLength( 0 ), m_cellMarkFound( false ),
+        m_remainingCells( 0 ), m_currentParagraph( new Paragraph ), m_remainingChars( 0 ),
+        m_sectionNumber( 0 ), m_subDocument( None ), m_parsingMode( Default )
 {
     if ( !isOk() )
         return;
@@ -167,6 +168,10 @@ bool Parser9x::parse()
             return false;
     }
 
+    //provide the headers mask to m_subDocumentHandler
+    if (m_headers) {
+        m_subDocumentHandler->headersMask(m_headers->headersMask());
+    }
     // start parsing the body
     if ( !parseBody() )
         return false;
@@ -354,8 +359,8 @@ void Parser9x::init()
     if (( m_fib.ccpFtn != 0 ) || ( m_fib.ccpEdn != 0 ))
         m_footnotes = new Footnotes97( m_table, m_fib );
 
-    if (( m_fib.fcPlcfbkf != 0 ) || ( m_fib.fcPlcfbkl != 0 ))
-        m_bookmark = new Bookmarks( m_table, m_fib );
+    if (( m_fib.lcbPlcfbkf != 0 ) || ( m_fib.lcbPlcfbkl != 0 ))
+        m_bookmarks = new Bookmarks( m_table, m_fib );
 
     if ( m_fib.ccpAtn != 0 ) {
         m_annotations = new Annotations( m_table, m_fib );
@@ -463,7 +468,9 @@ bool Parser9x::parseBody()
     if ( !sep )
         sep = new Word97::SEP(); // don't pass 0 pointers in any case
     m_textHandler->sectionStart( sep ); // First section, starting at CP 0
+#ifdef WV2_DEBUG_SECTIONS
     sep->dump();
+#endif
     emitHeaderData( sep );
     sep = 0; // get rid of the huge SEP
 
@@ -512,12 +519,14 @@ void Parser9x::parseHelper( Position startPos )
                     string[ j ] &= 0x00ff;
                 }
             }
-            processPiece<XCHAR>( string, fc, limit, startPos ); // also takes care to delete [] string
+            // also takes care to delete [] string
+            processPiece<XCHAR>( string, fc, limit, startPos );
         }
         else {
             U8* string = new U8[ limit ];
             m_wordDocument->read( string, limit );
-            processPiece<U8>( string, fc, limit, startPos ); // also takes care to delete [] string
+            // also takes care to delete [] string
+            processPiece<U8>( string, fc, limit, startPos );
         }
         m_remainingChars -= limit;
         ++it;
@@ -554,6 +563,7 @@ void Parser9x::processPiece( String* string, U32 fc, U32 limit, const Position&
                 emitHeaderData( sep );
             }
             else {
+                //manual page break
                 m_textHandler->pageBreak();
             }
             break;
@@ -645,6 +655,7 @@ void Parser9x::processParagraph( U32 fc )
         if ( !m_tableRowStart ) {
             m_tableRowStart = new Position( m_currentParagraph->front().m_position );
             m_tableRowLength = 0;
+
 #ifdef WV2_DEBUG_TABLES
             wvlog << "Start of a table row: piece=" << m_tableRowStart->piece << " offset="
                     << m_tableRowStart->offset << endl;
@@ -708,6 +719,10 @@ void Parser9x::processParagraph( U32 fc )
                 processChunk( *it, chp, length, index, pcdIt.currentStart() );
                 index += length;
             }
+            //bookmark check for the next to last CP (paragraph mark)
+            if ( m_bookmarks ) {
+                emitBookmark( ( *it ).m_position.offset + limit );
+            }
         }
         m_textHandler->paragraphEnd();
 
@@ -742,32 +757,40 @@ void Parser9x::processChunk( const Chunk& chunk, SharedPtr<const Word97::CHP> ch
     //
     while ( length > 0 ) {
         U32 disruption = 0xffffffff; // "infinity"
+        U32 bkmk_length = 0; //num. of CPs enclosed in a bookmark
+
         if ( m_footnotes ) {
             U32 nextFtn = m_footnotes->nextFootnote();
             U32 nextEnd = m_footnotes->nextEndnote();
             disruption = nextFtn < nextEnd ? nextFtn : nextEnd;
 #ifdef WV2_DEBUG_FOOTNOTES
-            wvlog << "nextFtn=" << nextFtn << " nextEnd=" << nextEnd << " disruption="
-                  << disruption << " length=" << length << endl;
+            wvlog << "nextFtn=" << nextFtn << " nextEnd=" << nextEnd <<
+                     " disruption=" << disruption << " length=" << length << endl;
 #endif
-        } else if ( m_bookmark ) {
-            U32 nextBkf = m_bookmark->nextBookmarkStart();
-            U32 nextBkl = m_bookmark->nextBookmarkEnd();
-            disruption = nextBkf < nextBkl ? nextBkf : nextBkl;
+        } else if ( m_bookmarks ) {
+            U32 nextBkf = m_bookmarks->nextBookmarkStart();
+            U32 nextBkl = m_bookmarks->nextBookmarkEnd();
+            bkmk_length = nextBkl - nextBkf;
+
+            //it shouldn't be possible that (nextBkf < nextBkl)
+            Q_ASSERT (nextBkf <= nextBkl);
+            disruption = nextBkf;
+
 #ifdef WV2_DEBUG_BOOKMARK
-            wvlog << "nextBkf=" << nextBkf << " nextBkl=" << nextBkl << " disruption="
-                  << disruption << " length=" << length << endl;
+            wvlog << "nextBkf=" << nextBkf << " nextBkl=" << nextBkl << 
+                     " disruption=" << disruption << " length=" << length << endl;
 #endif
         }
         U32 startCP = currentStart + chunk.m_position.offset + index;
 
-        if ( disruption >= startCP && disruption < startCP + length ) {
+        if ( (disruption >= startCP) && (disruption < (startCP + length)) ) {
 
 #if defined WV2_DEBUG_FOOTNOTES || defined WV2_DEBUG_BOOKMARK
-            wvlog << "startCP=" << startCP << " len=" << length << " disruption=" 
-                  << disruption << endl;
+            wvlog << "startCP=" << startCP << " disruption=" << disruption << 
+		     " bkmk_length=" << bkmk_length << " length=" << length << endl;;
 #endif
             U32 disLen = disruption - startCP;
+            //there's something to be processed before the bookmark
             if ( disLen != 0 ) {
                 processRun( chunk, chp, disLen, index, currentStart );
             }
@@ -778,31 +801,48 @@ void Parser9x::processChunk( const Chunk& chunk, SharedPtr<const Word97::CHP> ch
                 m_customFootnote = chunk.m_text.substr(index, length);
                 emitFootnote( m_customFootnote, disruption, chp, length );
                 m_customFootnote = "";
-            } else if ( m_bookmark ) {
-                m_bookmarkText = chunk.m_text.substr(index, length);
-                emitBookmark( m_bookmarkText, disruption, chp, length );
-                m_bookmarkText = "";
+
+                length = 0;
+                index += length;
+            } 
+            else if ( m_bookmarks ) {
+
+                //TODO: there may be a number of bookmarks having different
+                //lengths at the current CP
+
+                //TODO: we need to handle a number of overlapping bookmarks in
+                //this chunk
+
+		bool ok;
+		BookmarkData data( m_bookmarks->bookmark( disruption, ok ) );
+
+                //TODO: a bookmark can cover text from several chunks
+                if ( !(bkmk_length <= length) ) {
+                    wvlog << "WARNING: bookmarks covering several chunks are not supported yet!";
+                    processRun( chunk, chp, length, index, currentStart );
+                    length = 0;
+                    index += length;
+                } else {
+                    m_textHandler->bookmarkStart( data );
+                    if (bkmk_length > 0) {
+                        processRun( chunk, chp, bkmk_length, index, currentStart );
+                        m_textHandler->bookmarkEnd( data );
+                        length -= bkmk_length;
+                        index += bkmk_length;
+                    }
+                }
             }
-            index += length;
-            length = 0;
         }
         else {
             //common case, no disruption at all (or the end of a disrupted chunk)
             //In case of custom footnotes do not add label to footnote body.
             if ( m_footnotes ) {
-                if (m_customFootnote.find(chunk.m_text.substr(index, length), 0) != 0)
+                if (m_customFootnote.find(chunk.m_text.substr(index, length), 0) != 0) {
                     processRun( chunk, chp, length, index, currentStart );
-            } else if ( m_bookmark ) {
-                if (m_bookmarkText.find(chunk.m_text.substr(index, length), 0) != 0) {
-                    processRun( chunk, chp, length, index, currentStart );
-                    if ( m_bookmark ) {
-                        m_bookmarkText = chunk.m_text.substr(index, length);
-                        emitBookmark( m_bookmarkText, 0, chp, length );
-                        m_bookmarkText = "";
-                    }
                 }
+            } else {
+                processRun( chunk, chp, length, index, currentStart );
             }
-
             break;   // should be faster than messing with length...
         }
     }
@@ -918,21 +958,26 @@ void Parser9x::emitFootnote( UString characters, U32 globalCP, SharedPtr<const W
         m_textHandler->footnoteFound( data.type, characters, chp, make_functor( *this, &Parser9x::parseFootnote, data ));
 }
 
-void Parser9x::emitBookmark( UString characters, U32 globalCP, SharedPtr<const Word97::CHP> chp, U32 length )
+void Parser9x::emitBookmark( U32 globalCP )
 {
-    if ( !m_bookmark ) {
-        wvlog << "Bug: Found a bookmark, but m_bookmark == 0!" << endl;
-        return;
-    }
-    bool ok;
-    BookmarkData data( m_bookmark->bookmark( globalCP, ok ) );
+    bool ok = false;
+    BookmarkData data( m_bookmarks->bookmark( globalCP, ok ) );
+
+    //TODO: handle bookmarks marking a text range between paragraphs in this
+    //special case
+
+    //there might be more bookmarks for the current CP
+    while (ok) {
+        if ((data.limCP - data.startCP) > 0) {
+            wvlog << "WARNING: bookmarks marking a text range between paragraphs not supported!";
+        } else {
+            m_textHandler->bookmarkStart( data );
+        }
+        data = m_bookmarks->bookmark( globalCP, ok );
 
-    if (ok) {
 #ifdef WV2_DEBUG_BOOKMARK
-    wvlog << "Bookmark found: CP=" << globalCP << endl;
+        wvlog << "Bookmark found: CP=" << globalCP << endl;
 #endif
-        // We introduce the name of the bookmark here with data.name
-        m_textHandler->bookmarkFound( characters, data.name, chp);
     }
 }
 
@@ -1050,7 +1095,7 @@ void Parser9x::emitPictureData( SharedPtr<const Word97::CHP> chp )
 
     SharedPtr<const Word97::PICF> sharedPicf( picf );
     PictureData data( offset, sharedPicf );
-    m_textHandler->inlineObjectFound(data, chp);
+    m_textHandler->inlineObjectFound(data);
 
     //NOTE: the depreciated approach to parse an inline object follows
 //     PictureFunctor fnct = make_functor( *this, &Parser9x::parsePicture,
@@ -1080,13 +1125,13 @@ void Parser9x::parseHeader( const HeaderData& data, unsigned char mask )
 #endif
     if ( length < 1 ) {
 #ifdef WV2_DEBUG_HEADERS
-        wvlog << "Warning: Didn't find a valid CPs for this header -- faking it" << endl;
+        wvlog << "Warning: Didn't find valid CPs for this header/footer -- ignoring it" << endl;
 #endif
-        m_subDocumentHandler->headerStart( static_cast<HeaderData::Type>( mask ) );
-        SharedPtr<const ParagraphProperties> sharedProps( new ParagraphProperties );
-        m_textHandler->paragraphStart( sharedProps );
-        m_textHandler->paragraphEnd();
-        m_subDocumentHandler->headerEnd();
+//         m_subDocumentHandler->headerStart( static_cast<HeaderData::Type>( mask ) );
+//         SharedPtr<const ParagraphProperties> sharedProps( new ParagraphProperties );
+//         m_textHandler->paragraphStart( sharedProps );
+//         m_textHandler->paragraphEnd();
+//         m_subDocumentHandler->headerEnd();
         return;
     }
     else if ( length > 1 )
