@@ -46,6 +46,8 @@
 #include "MsooXmlSchemas.h"
 #include "MsooXmlReader.h"
 
+#include "pole.h"
+
 #include <KoOdfReadStore.h>
 #include <styles/KoCharacterStyle.h>
 #include <KoXmlReader.h>
@@ -83,6 +85,7 @@ const char* const MSOOXML::ContentTypes::wordEndnotes =         "application/vnd
 const char* const MSOOXML::ContentTypes::wordFontTable =        "application/vnd.openxmlformats-officedocument.wordprocessingml.fontTable+xml";
 const char* const MSOOXML::ContentTypes::wordWebSettings =      "application/vnd.openxmlformats-officedocument.wordprocessingml.webSettings+xml";
 const char* const MSOOXML::ContentTypes::wordTemplate =         "application/vnd.openxmlformats-officedocument.wordprocessingml.template.main+xml";
+const char* const MSOOXML::ContentTypes::wordComments =         "application/vnd.openxmlformats-officedocument.wordprocessingml.comments+xml";
 
 // presentationml-specific content types
 const char* const MSOOXML::ContentTypes::presentationDocument =      "application/vnd.openxmlformats-officedocument.presentationml.presentation.main+xml";
@@ -94,6 +97,7 @@ const char* const MSOOXML::ContentTypes::presentationNotes =         "applicatio
 const char* const MSOOXML::ContentTypes::presentationTableStyles =   "application/vnd.openxmlformats-officedocument.presentationml.tableStyles+xml";
 const char* const MSOOXML::ContentTypes::presentationProps =         "application/vnd.openxmlformats-officedocument.presentationml.presProps+xml";
 const char* const MSOOXML::ContentTypes::presentationViewProps =     "application/vnd.openxmlformats-officedocument.presentationml.viewProps+xml";
+const char* const MSOOXML::ContentTypes::presentationComments =      "application/vnd.openxmlformats-officedocument.presentationml.comments+xml";
 
 // spreadsheetml-specific content types
 const char* const MSOOXML::ContentTypes::spreadsheetDocument =        "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet.main+xml";
@@ -103,6 +107,7 @@ const char* const MSOOXML::ContentTypes::spreadsheetWorksheet =       "applicati
 const char* const MSOOXML::ContentTypes::spreadsheetCalcChain =       "application/vnd.openxmlformats-officedocument.spreadsheetml.calcChain+xml";
 const char* const MSOOXML::ContentTypes::spreadsheetSharedStrings =   "application/vnd.openxmlformats-officedocument.spreadsheetml.sharedStrings+xml";
 const char* const MSOOXML::ContentTypes::spreadsheetTemplate =        "application/vnd.openxmlformats-officedocument.spreadsheetml.template.main+xml";
+const char* const MSOOXML::ContentTypes::spreadsheetComments =        "application/vnd.openxmlformats-officedocument.spreadsheetml.comments+xml";
 
 //generic namespaces
 const char* const MSOOXML::Schemas::dublin_core =                                "http://purl.org/dc/elements/1.1/";
@@ -243,27 +248,115 @@ QIODevice* Utils::openDeviceForFile(const KZip* zip, QString& errorMessage, cons
     return f->createDevice();
 }
 
+#define BLOCK_SIZE 4096
+static KoFilter::ConversionStatus copyOle(QString& errorMessage,
+                                    const QString sourceName, KoStore *outputStore,
+                                    const QString& destinationName, const KZip* zip)
+{
+    KoFilter::ConversionStatus status = KoFilter::OK;
+
+    QIODevice* inputDevice = Utils::openDeviceForFile(zip, errorMessage, sourceName, status);
+    inputDevice->open(QIODevice::ReadOnly);
+
+    POLE::Storage storage(inputDevice);
+    if (!storage.open()) {
+        kDebug(30513) << "Cannot open " << sourceName;
+        return KoFilter::WrongFormat;
+    }
+
+    std::list<std::string> lista = storage.entries();
+    std::string oleType = "Contents";
+
+    for (std::list<std::string>::iterator it = lista.begin(); it != lista.end(); ++it)  {
+        //qDebug() << "ENTRY " << (*it).c_str();
+        if (QString((*it).c_str()).contains("Ole10Native")) {
+            oleType = "Ole10Native";
+        }
+        else if (QString((*it).c_str()).contains("CONTENTS")) {
+            oleType = "CONTENTS";
+        }
+    }
+
+    POLE::Stream stream(&storage, oleType);
+    QByteArray array;
+    array.resize(stream.size());
+
+    unsigned long r = stream.read((unsigned char*)array.data(), stream.size());
+    if (r != stream.size()) {
+        kError(30513) << "Error while reading from stream";
+        return KoFilter::WrongFormat;
+    }
+
+    if (oleType == "Contents" || oleType == "Ole10Native") {
+     // Removing first 4 bytes which are the size
+        array = array.right(array.length() - 4);
+    }
+
+    // Uncomment to write any ole file for testing
+    //POLE::Stream streamTemp(&storage, "Ole");
+    //QByteArray arrayTemp;
+    //arrayTemp.resize(streamTemp.size());
+    //streamTemp.read((unsigned char*)arrayTemp.data(), streamTemp.size());
+    //QFile file("olething.ole");
+    //file.open(QIODevice::WriteOnly);
+    //QDataStream out(&file);
+    //out.writeRawData(arrayTemp.data(), arrayTemp.length());
+
+    kDebug() << "mode:" << outputStore->mode();
+    if (!outputStore->open(destinationName)) {
+        errorMessage = i18n("Could not open entry \"%1\" for writing.", destinationName);
+        return KoFilter::CreationError;
+    }
+
+    QByteArray array2;
+    while (true) {
+        array2 = array.left(BLOCK_SIZE);
+        array = array.right(array.size() - array2.size());
+        const qint64 in = array2.size();
+        if (in <= 0) {
+            break;
+        }
+        char *block = array2.data();
+        if (in != outputStore->write(block, in)) {
+            errorMessage = i18n("Could not write block");
+            status = KoFilter::CreationError;
+            break;
+        }
+    }
+    outputStore->close();
+    delete inputDevice;
+    inputDevice = 0;
+    return status;
+}
+#undef BLOCK_SIZE
+
 #define BLOCK_SIZE 4096
 KoFilter::ConversionStatus Utils::copyFile(const KZip* zip, QString& errorMessage,
                                            const QString& sourceName, KoStore *outputStore,
-                                           const QString& destinationName)
+                                           const QString& destinationName, bool oleType)
 {
     if (outputStore->hasFile(destinationName)) {
         return KoFilter::OK;
     }
 
     KoFilter::ConversionStatus status;
-    std::auto_ptr<QIODevice> inputDevice(Utils::openDeviceForFile(zip, errorMessage, sourceName, status));
+    if (oleType) {
+        status = copyOle(errorMessage, sourceName, outputStore, destinationName, zip);
+        return status;
+    }
+
+    std::auto_ptr<QIODevice> inputDevice = std::auto_ptr<QIODevice>(Utils::openDeviceForFile(zip, errorMessage, sourceName, status));
+
     if (!inputDevice.get()) {
         return status;
     }
+
     kDebug() << "mode:" << outputStore->mode();
     if (!outputStore->open(destinationName)) {
         errorMessage = i18n("Could not open entry \"%1\" for writing.", destinationName);
         return KoFilter::CreationError;
     }
     status = KoFilter::OK;
-//    QIODevice* outputDevice = outputStore->device();
     char block[BLOCK_SIZE];
     while (true) {
         const qint64 in = inputDevice->read(block, BLOCK_SIZE);
