@@ -1,5 +1,10 @@
 /* This file is part of the KDE project
- * Copyright (C) 2007 Thorsten Zachmann <zachmann@kde.org>
+ * Copyright (C) 2007-2009 Thorsten Zachmann <zachmann@kde.org>
+ * Copyright (C) 2008 Jim Courtiau <jeremy.courtiau@gmail.com>
+ * Copyright (C) 2009 Alexia Allanic <alexia_allanic@yahoo.fr>
+ * Copyright (C) 2009 Jean-Nicolas Artaud <jeannicolasartaud@gmail.com>
+ * Copyright (C) 2009 Jérémy Lugagne <jejewindsurf@hotmail.com>
+ * Copyright (C) 2009 Johann Hingue <yoan1703@hotmail.fr>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -19,21 +24,26 @@
 #ifndef KPRPRESENTATIONTOOL_H
 #define KPRPRESENTATIONTOOL_H
 
-#include <KoTool.h>
+#include <KoToolBase.h>
 
-#include <QList>
+#include <QSet>
 
+class QFrame;
 class KoEventAction;
 class KPrViewModePresentation;
+class KPrPresentationToolWidget;
+class KPrPresentationStrategyBase;
+class KPrPresentationToolAdaptor;
 
 /// The tool used for presentations
-class KPrPresentationTool : public KoTool {
+class KPrPresentationTool : public KoToolBase
+{
     Q_OBJECT
 public:
-    explicit KPrPresentationTool( KPrViewModePresentation & viewMode );
+    explicit KPrPresentationTool( KPrViewModePresentation &viewMode );
     ~KPrPresentationTool();
 
-    bool wantsAutoScroll();
+    bool wantsAutoScroll() const;
 
     void paint( QPainter &painter, const KoViewConverter &converter );
 
@@ -45,15 +55,30 @@ public:
     void keyReleaseEvent( QKeyEvent *event );
     void wheelEvent( KoPointerEvent * event );
 
+    KPrPresentationStrategyBase *strategy();
+    KPrViewModePresentation & viewModePresentation();
+
 public slots:
-    void activate( bool temporary = false );
+    virtual void activate(ToolActivation toolActivation, const QSet<KoShape*> &shapes);
     void deactivate();
+    void highlightPresentation();
+    void drawOnPresentation();
+    void blackPresentation();
+    void normalPresentation();
 
 private:
     void finishEventActions();
+    void switchStrategy( KPrPresentationStrategyBase * strategy );
+    bool eventFilter( QObject * obj, QEvent * event );
 
     KPrViewModePresentation & m_viewMode;
-    QList<KoEventAction *> m_eventActions;
+    QSet<KoEventAction *> m_eventActions;
+
+    KPrPresentationToolWidget * m_presentationToolWidget;
+    QFrame * m_frame;
+    KPrPresentationStrategyBase * m_strategy;
+    KPrPresentationToolAdaptor *m_bus;
+    friend class KPrPresentationStrategyBase;
 };
 
 #endif /* KPRPRESENTATIONTOOL_H */
