@@ -64,7 +64,6 @@ public:
      */
     bool isPresentationRunning() const;
 
-
 public slots:
     /**
      * Activate the presentation view mode
@@ -93,6 +92,10 @@ protected slots:
     void configureSlideShow();
     void configurePresenterView();
     void exportToHtml();
+    void insertPictures();
+    void drawOnPresentation();
+    void highlightPresentation();
+    void blackPresentation();
 
 private:
     KActionMenu *m_actionStartPresentation;
@@ -101,6 +104,10 @@ private:
     KAction *m_actionViewModeNotes;
     KAction *m_actionCreateCustomSlideShowsDialog;
     KAction *m_actionExportHtml;
+    KAction *m_actionInsertPictures;
+    KAction *m_actionDrawOnPresentation;
+    KAction *m_actionHighlightPresentation;
+    KAction *m_actionBlackPresentation;
 
     KPrViewModePresentation *m_presentationMode;
     KoPAViewMode *m_normalMode;
