@@ -36,6 +36,7 @@ class KoStore;
 class KoOdfLoadingContext;
 class KoEmbeddedDocumentSaver;
 class KoStyleManager;
+class KoResourceManager;
 
 class KCompletion;
 
@@ -197,11 +198,6 @@ public:
      */
     void setDefaultRowHeight(double height);
 
-    /**
-     * \ingroup OpenDocument
-     */
-    void saveOdfSettings(KoXmlWriter &settingsWriter);
-
     /**
      * \ingroup OpenDocument
      */
@@ -285,7 +281,7 @@ public:
      * @return index of @p sheet in this map
      * @return @c 0 if the index exceeds the list boundaries
      */
-    int indexOf( Sheet* sheet ) const;
+    int indexOf(Sheet* sheet) const;
 
     /**
      * @return the list of sheets in this map
@@ -355,6 +351,14 @@ public:
      */
     void addDamage(Damage* damage);
 
+    /**
+     * Return a pointer to the resource manager associated with the
+     * document. The resource manager contains
+     * document wide resources * such as variable managers, the image
+     * collection and others.
+     * @see KoCanvasBase::resourceManager()
+     */
+    KoResourceManager *resourceManager() const;
 public Q_SLOTS:
     /**
      * \ingroup Damages
