@@ -11,7 +11,7 @@
    Copyright 1999-2000 Simon Hausmann <hausmann@kde.org>
    Copyright 1999 David Faure <faure@kde.org>
    Copyright 1998-2000 Torben Weis <weis@kde.org>
-
+   
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
@@ -44,6 +44,7 @@
 //Added by qt3to4:
 #include <QList>
 #include <QPainter>
+#include <QGraphicsItem>
 
 #include <kstandarddirs.h>
 #include <kdebug.h>
@@ -74,11 +75,12 @@
 #include "BindingManager.h"
 #include "CalculationSettings.h"
 #include "Canvas.h"
+#include "CanvasItem.h"
 #include "DependencyManager.h"
 #include "Factory.h"
 #include "Formula.h"
+#include "Function.h"
 #include "FunctionModuleRegistry.h"
-#include "Functions.h"
 #include "HeaderFooter.h"
 #include "LoadingInfo.h"
 #include "Localization.h"
@@ -93,6 +95,7 @@
 #include "Util.h"
 #include "View.h"
 #include "SheetAccessModel.h"
+#include "BindingModel.h"
 
 // chart shape
 #include "plugins/chartshape/ChartShape.h"
@@ -118,12 +121,14 @@ public:
     static int s_docId;
 
     // document properties
-    bool configLoadFromFile : 1;
+    bool configLoadFromFile       : 1;
     QStringList spellListIgnoreAll;
     SavedDocParts savedDocParts;
     SheetAccessModel *sheetAccessModel;
 };
 
+Q_DECLARE_METATYPE(QPointer<QAbstractItemModel>)
+
 /*****************************************************************************
  *
  * Doc
@@ -238,6 +243,11 @@ KoView* Doc::createViewInstance(QWidget* parent)
     return new View(parent, this);
 }
 
+QGraphicsItem *Doc::createCanvasItem()
+{
+    return new CanvasItem(this);
+}
+
 bool Doc::saveChildren(KoStore* _store)
 {
     return map()->saveChildren(_store);
@@ -289,6 +299,17 @@ QDomDocument Doc::saveXML()
     }
 
     QDomElement e = map()->save(doc);
+    if (!views().isEmpty()) { // no view if embedded document
+        // Save visual info for the first view, such as active sheet and active cell
+        // It looks like a hack, but reopening a document creates only one view anyway (David)
+        View *const view = static_cast<View*>(views().first());
+        Canvas *const canvas = view->canvasWidget();
+        e.setAttribute("activeTable",  canvas->activeSheet()->sheetName());
+        e.setAttribute("markerColumn", view->selection()->marker().x());
+        e.setAttribute("markerRow",    view->selection()->marker().y());
+        e.setAttribute("xOffset",      canvas->xOffset());
+        e.setAttribute("yOffset",      canvas->yOffset());
+    }
     spread.appendChild(e);
 
     setModified(false);
@@ -402,7 +423,37 @@ void Doc::saveOdfSettings(KoXmlWriter &settingsWriter)
     settingsWriter.startElement("config:config-item-map-indexed");
     settingsWriter.addAttribute("config:name", "Views");
     settingsWriter.startElement("config:config-item-map-entry");
-    map()->saveOdfSettings(settingsWriter);
+    settingsWriter.addConfigItem("ViewId", QString::fromLatin1("View1"));
+    if (!views().isEmpty()) { // no view if embedded document
+        // Save visual info for the first view, such as active sheet and active cell
+        // It looks like a hack, but reopening a document creates only one view anyway (David)
+        View *const view = static_cast<View*>(views().first());
+        // save current sheet selection before to save marker, otherwise current pos is not saved
+        view->saveCurrentSheetSelection();
+        //<config:config-item config:name="ActiveTable" config:type="string">Feuille1</config:config-item>
+        if (Sheet *sheet = view->activeSheet()) {
+            settingsWriter.addConfigItem("ActiveTable", sheet->sheetName());
+        }
+    }
+    //<config:config-item-map-named config:name="Tables">
+    settingsWriter.startElement("config:config-item-map-named");
+    settingsWriter.addAttribute("config:name", "Tables");
+    foreach (Sheet *sheet, map()->sheetList()) {
+        settingsWriter.startElement("config:config-item-map-entry");
+        settingsWriter.addAttribute("config:name", sheet->sheetName());
+        if (!views().isEmpty()) {
+            View *const view = static_cast<View*>(views().first());
+            QPoint marker = view->markerFromSheet(sheet);
+            QPointF offset = view->offsetFromSheet(sheet);
+            settingsWriter.addConfigItem("CursorPositionX", marker.x() - 1);
+            settingsWriter.addConfigItem("CursorPositionY", marker.y() - 1);
+            settingsWriter.addConfigItem("xOffset", offset.x());
+            settingsWriter.addConfigItem("yOffset", offset.y());
+        }
+        sheet->saveOdfSettings(settingsWriter);
+        settingsWriter.endElement();
+    }
+    settingsWriter.endElement();
     settingsWriter.endElement();
     settingsWriter.endElement();
 }
@@ -465,6 +516,21 @@ bool Doc::loadOdf(KoOdfReadStore & odfStore)
         loadOdfSettings(odfStore.settingsDoc());
     }
     initConfig();
+
+    //update plugins that rely on bindings, as loading order can mess up the data of the plugins
+    SheetAccessModel* sheetModel = sheetAccessModel();
+    QList< Sheet* > sheets = map()->sheetList();
+    Q_FOREACH( Sheet* sheet, sheets ){
+        // This region contains the entire sheet
+        const QRect region (0, 0, KS_colMax - 1, KS_rowMax - 1);
+        QModelIndex index = sheetModel->index( 0, map()->indexOf( sheet ) );
+          QVariant bindingModelValue = sheetModel->data( index , Qt::DisplayRole );
+          BindingModel *curBindingModel = dynamic_cast< BindingModel* >( qvariant_cast< QPointer< QAbstractItemModel > >( bindingModelValue ).data() );
+          if ( curBindingModel ){
+              curBindingModel->emitDataChanged( region );
+          }
+    }
+
     emit sigProgress(-1);
 
     //display loading time
@@ -494,7 +560,7 @@ bool Doc::loadXML(const KoXmlDocument& doc, KoStore*)
     if (map()->syntaxVersion() > CURRENT_SYNTAX_VERSION) {
         int ret = KMessageBox::warningContinueCancel(
                       0, i18n("This document was created with a newer version of KSpread (syntax version: %1)\n"
-                              "When you open it with this version of KSpread, some information may be lost.",map()->syntaxVersion()),
+                              "When you open it with this version of KSpread, some information may be lost.", map()->syntaxVersion()),
                       i18n("File Format Mismatch"), KStandardGuiItem::cont());
         if (ret == KMessageBox::Cancel) {
             setErrorMessage("USER_CANCELED");
@@ -657,7 +723,7 @@ void Doc::loadPaper(KoXmlElement const & paper)
 
     foreach(Sheet* sheet, map()->sheetList()) {
         sheet->print()->headerFooter()->setHeadFootLine(hleft, hcenter, hright,
-                                                        fleft, fcenter, fright);
+                fleft, fcenter, fright);
     }
 }
 
@@ -668,7 +734,7 @@ bool Doc::completeLoading(KoStore* store)
     setModified(false);
     bool ok = map()->completeLoading(store);
 
-  kDebug(36001) <<"------------------------ COMPLETION DONE --------------------";
+    kDebug(36001) << "------------------------ COMPLETION DONE --------------------";
     return ok;
 }
 
@@ -694,18 +760,22 @@ QStringList Doc::spellListIgnoreAll() const
 }
 
 void Doc::paintContent(QPainter& painter, const QRect& rect)
+{
+    paintContent(painter, rect, 0);
+}
+
+void Doc::paintContent(QPainter& painter, const QRect& rect, Sheet* _sheet)
 {
     if (rect.isEmpty()) {
         return;
     }
-    Sheet *const sheet = d->map->sheet(0);
+    Sheet *const sheet = _sheet ? _sheet : d->map->sheet(0);
 
     const KoPageLayout pageLayout = sheet->printSettings()->pageLayout();
     QPixmap thumbnail(pageLayout.width, pageLayout.height);
     thumbnail.fill(Qt::white);
 
     SheetView sheetView(sheet);
-    sheetView.setPaintDevice(&thumbnail);
 
     const qreal zoom = sheet->printSettings()->zoom();
     KoZoomHandler zoomHandler;
@@ -715,7 +785,8 @@ void Doc::paintContent(QPainter& painter, const QRect& rect)
     sheetView.setPaintCellRange(sheet->print()->cellRange(1)); // first page
 
     QPainter pixmapPainter(&thumbnail);
-    sheetView.paintCells(&thumbnail, pixmapPainter, QRect(0, 0, pageLayout.width, pageLayout.height), QPointF(0,0));
+    pixmapPainter.setClipRect(QRect(QPoint(0, 0), thumbnail.size()));
+    sheetView.paintCells(pixmapPainter, QRect(0, 0, pageLayout.width, pageLayout.height), QPointF(0,0));
 
     // The pixmap gets scaled to fit the rectangle.
     painter.drawPixmap(rect & QRect(0, 0, 100, 100), thumbnail);
