@@ -9,7 +9,8 @@
    Copyright 1999-2003 David Faure <faure@kde.org>
    Copyright 1999-2001 Simon Hausmann <hausmann@kde.org>
    Copyright 1998-2000 Torben Weis <weis@kde.org>
-
+   Copyright 2010 Boudewijn Rempt <boud@kogmbh.com>
+   
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
@@ -84,7 +85,7 @@
 // KOffice includes
 #include <KoGlobal.h>
 #include <KoDpi.h>
-#include <KoCanvasController.h>
+#include <KoCanvasControllerWidget.h>
 #include <KoMainWindow.h>
 #include <KoOdfLoadingContext.h>
 #include <KoOdfReadStore.h>
@@ -119,6 +120,7 @@
 #include "Canvas.h"
 #include "Global.h"
 #include "Headers.h"
+#include "HeaderWidgets.h"
 #include "Localization.h"
 #include "Map.h"
 #include "NamedAreaManager.h"
@@ -139,6 +141,7 @@
 #include "PrintJob.h"
 
 // commands
+#include "commands/CopyCommand.h"
 #include "commands/DefinePrintRangeCommand.h"
 #include "commands/SheetCommands.h"
 #include "commands/Undo.h"
@@ -152,7 +155,6 @@
 
 // ui
 #include "ui/CellView.h"
-#include "ui/Editors.h"
 #include "ui/MapViewModel.h"
 #include "ui/RightToLeftPaintingStrategy.h"
 #include "ui/SheetView.h"
@@ -183,9 +185,9 @@ public:
     KoCanvasController* canvasController;
     KoZoomController* zoomController;
     KoZoomHandler* zoomHandler;
-    RowHeader *rowHeader;
-    ColumnHeader *columnHeader;
-    SelectAllButton* selectAllButton;
+    RowHeaderWidget *rowHeader;
+    ColumnHeaderWidget *columnHeader;
+    SelectAllButtonWidget* selectAllButton;
     QScrollBar *horzScrollBar;
     QScrollBar *vertScrollBar;
     TabBar *tabBar;
@@ -279,7 +281,7 @@ void View::Private::initActions()
     KActionCollection* ac = view->actionCollection();
 
     // -- sheet/workbook actions --
-    actions->sheetProperties  = new KAction(i18n("Sheet Properties"), view);
+    actions->sheetProperties  = new KAction(i18n("Sheet Properties..."), view);
     ac->addAction("sheetProperties", actions->sheetProperties);
     connect(actions->sheetProperties, SIGNAL(triggered(bool)), view, SLOT(sheetProperties()));
     actions->sheetProperties->setToolTip(i18n("Modify current sheet's properties"));
@@ -675,11 +677,12 @@ void View::initView()
 
     // Setup the Canvas and its controller.
     d->canvas = new Canvas(this);
-    d->canvasController = new KoCanvasController(this);
+    KoCanvasControllerWidget *canvasController = new KoCanvasControllerWidget(this);
+    d->canvasController = canvasController;
     d->canvasController->setCanvas(d->canvas);
     d->canvasController->setCanvasMode(KoCanvasController::Spreadsheet);
-    d->canvasController->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
-    d->canvasController->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
+    canvasController->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
+    canvasController->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
 
     // Setup the map model.
     d->mapViewModel = new MapViewModel(d->doc->map(), d->canvas, this);
@@ -720,7 +723,7 @@ void View::initView()
         shell()->createDockWidget(&toolBoxFactory);
 
         // Setup the tool options dock widget manager.
-        connect(d->canvasController, SIGNAL(toolOptionWidgetsChanged(const QMap<QString, QWidget *> &, QWidget*)),
+        connect(canvasController, SIGNAL(toolOptionWidgetsChanged(const QMap<QString, QWidget *> &, QWidget*)),
                 shell()->dockerManager(), SLOT(newOptionWidgets(const  QMap<QString, QWidget *> &, QWidget*)));
     }
     // Setup the zoom controller.
@@ -731,11 +734,11 @@ void View::initView()
     connect(d->zoomController, SIGNAL(zoomChanged(KoZoomMode::Mode, qreal)),
             this, SLOT(viewZoom(KoZoomMode::Mode, qreal)));
 
-    d->columnHeader = new ColumnHeader(this, d->canvas, this);
-    d->rowHeader = new RowHeader(this, d->canvas , this);
+    d->columnHeader = new ColumnHeaderWidget(this, d->canvas, this);
+    d->rowHeader = new RowHeaderWidget(this, d->canvas , this);
     d->columnHeader->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Minimum);
     d->rowHeader->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Expanding);
-    d->selectAllButton = new SelectAllButton(d->canvas, d->selection);
+    d->selectAllButton = new SelectAllButtonWidget(d->canvas);
     d->selectAllButton->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
 
     d->canvas->setFocusPolicy(Qt::StrongFocus);
@@ -747,7 +750,7 @@ void View::initView()
     // Vert. Scroll Bar
     d->calcLabel  = 0;
     d->vertScrollBar = new QScrollBar(this);
-    d->canvasController->setVerticalScrollBar(d->vertScrollBar);
+    canvasController->setVerticalScrollBar(d->vertScrollBar);
     d->vertScrollBar->setOrientation(Qt::Vertical);
     d->vertScrollBar->setSingleStep(60);  //just random guess based on what feels okay
     d->vertScrollBar->setPageStep(60);  //This should be controlled dynamically, depending on how many rows are shown
@@ -759,7 +762,7 @@ void View::initView()
     d->tabBar = new TabBar(0);
     d->tabScrollBarLayout->addWidget(d->tabBar, 0, 0);
     d->horzScrollBar = new QScrollBar(0);
-    d->canvasController->setHorizontalScrollBar(d->horzScrollBar);
+    canvasController->setHorizontalScrollBar(d->horzScrollBar);
     d->tabScrollBarLayout->addWidget(d->horzScrollBar, 0, 1, 2, 1, Qt::AlignVCenter);
 
     d->horzScrollBar->setOrientation(Qt::Horizontal);
@@ -784,7 +787,7 @@ void View::initView()
     d->viewLayout->addWidget(d->selectAllButton, 1, 0);
     d->viewLayout->addWidget(d->columnHeader, 1, 1, 1, 1);
     d->viewLayout->addWidget(d->rowHeader, 2, 0);
-    d->viewLayout->addWidget(d->canvasController, 2, 1);
+    d->viewLayout->addWidget(canvasController, 2, 1);
     d->viewLayout->addWidget(d->vertScrollBar, 1, 2, 2, 1, Qt::AlignHCenter);
     d->viewLayout->addWidget(bottomPart, 3, 0, 1, 2);
     d->viewLayout->setColumnMinimumWidth(2, extent);
@@ -800,8 +803,8 @@ void View::initView()
 
     // signal slot
     connect(d->canvas, SIGNAL(documentSizeChanged(const QSize&)),
-            d->canvasController, SLOT(setDocumentSize(const QSize&)));
-    connect(d->canvasController, SIGNAL(moveDocumentOffset(const QPoint&)),
+            d->canvasController->proxyObject, SLOT(updateDocumentSize(const QSize&)));
+    connect(d->canvasController->proxyObject, SIGNAL(moveDocumentOffset(const QPoint&)),
             d->canvas, SLOT(setDocumentOffset(const QPoint&)));
     connect(d->canvas->shapeManager(), SIGNAL(selectionChanged()),
             this, SLOT(shapeSelectionChanged()));
@@ -822,12 +825,12 @@ KoCanvasController* View::canvasController() const
     return d->canvasController;
 }
 
-ColumnHeader* View::columnHeader()const
+ColumnHeaderWidget* View::columnHeader()const
 {
     return d->columnHeader;
 }
 
-RowHeader* View::rowHeader()const
+RowHeaderWidget* View::rowHeader()const
 {
     return d->rowHeader;
 }
@@ -872,7 +875,6 @@ SheetView* View::sheetView(const Sheet* sheet) const
     if (!d->sheetViews.contains(sheet)) {
         kDebug(36004) << "View: Creating SheetView for" << sheet->sheetName();
         d->sheetViews.insert(sheet, new SheetView(sheet));
-        d->sheetViews[ sheet ]->setPaintDevice(d->canvas);
         d->sheetViews[ sheet ]->setViewConverter(zoomHandler());
         connect(d->sheetViews[ sheet ], SIGNAL(visibleSizeChanged(const QSizeF&)),
                 d->canvas, SLOT(setDocumentSize(const QSizeF&)));
@@ -1210,6 +1212,9 @@ void View::setActiveSheet(Sheet* sheet, bool updateSheet)
         d->vertScrollBar->setValue(offset.y());
     }
 
+    // tell the resource manager of the newly active page
+    d->canvas->resourceManager()->setResource(KoCanvasResource::CurrentPage, QVariant(sheet->map()->indexOf(sheet) + 1));
+
     // Always repaint the visible cells.
     d->canvas->update();
     d->rowHeader->update();
@@ -1431,7 +1436,10 @@ void View::copyAsText()
 {
     if (!d->activeSheet)
         return;
-    d->activeSheet->copyAsText(selection());
+    QMimeData* mimeData = new QMimeData();
+    mimeData->setText(CopyCommand::saveAsPlainText(*selection()));
+
+    QApplication::clipboard()->setMimeData(mimeData);
 }
 
 
@@ -1455,10 +1463,10 @@ void View::toggleProtectDoc(bool mode)
     bool success;
     if (mode) {
         success = doc()->map()->showPasswordDialog(this, ProtectableObject::Lock,
-                                                   i18n("Protect Document"));
+                  i18n("Protect Document"));
     } else {
         success = doc()->map()->showPasswordDialog(this, ProtectableObject::Unlock,
-                                                   i18n("Unprotect Document"));
+                  i18n("Unprotect Document"));
     }
     if (!success) {
         d->actions->protectDoc->setChecked(!mode);
@@ -1478,10 +1486,10 @@ void View::toggleProtectSheet(bool mode)
     bool success;
     if (mode) {
         success = activeSheet()->showPasswordDialog(this, ProtectableObject::Lock,
-                                                    i18n("Protect Sheet"));
+                  i18n("Protect Sheet"));
     } else {
         success = activeSheet()->showPasswordDialog(this, ProtectableObject::Unlock,
-                                                    i18n("Unprotect Sheet"));
+                  i18n("Unprotect Sheet"));
     }
     if (!success) {
         d->actions->protectSheet->setChecked(!mode);
@@ -1636,7 +1644,7 @@ void View::keyPressEvent(QKeyEvent *event)
 
 int View::leftBorder() const
 {
-    return d->rowHeader->width();
+    return (int)(((RowHeader*)d->rowHeader)->width());
 }
 
 int View::rightBorder() const
@@ -1646,7 +1654,7 @@ int View::rightBorder() const
 
 int View::topBorder() const
 {
-    return d->columnHeader->height();
+    return (int)(((ColumnHeader*)d->columnHeader)->height());
 }
 
 int View::bottomBorder() const
@@ -1672,8 +1680,10 @@ void View::paperLayoutDlg()
 
     KoPageLayout pl = print->settings()->pageLayout();
 
+
+/*
     const HeaderFooter *const headerFooter = print->headerFooter();
-/*    HeadFoot hf;
+    HeadFoot hf;
     hf.headLeft  = headerFooter->localizeHeadFootLine(headerFooter->headLeft());
     hf.headRight = headerFooter->localizeHeadFootLine(headerFooter->headRight());
     hf.headMid   = headerFooter->localizeHeadFootLine(headerFooter->headMid());
@@ -2044,9 +2054,9 @@ void View::handleDamages(const QList<Damage*>& damages)
             CellDamage* cellDamage = static_cast<CellDamage*>(damage);
             kDebug(36007) << "Processing\t" << *cellDamage;
             Sheet* const damagedSheet = cellDamage->sheet();
-            const Region region = cellDamage->region();
 
             if (cellDamage->changes() & CellDamage::Appearance) {
+                const Region& region = cellDamage->region();
                 sheetView(damagedSheet)->invalidateRegion(region);
                 paintMode = Everything;
             }
