@@ -31,8 +31,6 @@
 
 // KSpread
 #include "CellToolBase.h"
-#include "Editors.h"
-#include "CharSelectDia.h"
 
 // dialogs
 #include "dialogs/FindDialog.h"
@@ -50,8 +48,11 @@ class QUndoCommand;
 class KFind;
 class KReplace;
 
+class CharSelectDia;
+
 namespace KSpread
 {
+class CellEditor;
 class CellToolOptionWidget;
 class Sheet;
 
@@ -73,6 +74,7 @@ public:
     bool initialized;
     // For "Selection List..." action
     QMenu* popupListChoose;
+    Editor lastEditorWithFocus;
 
     // Find and Replace context. We remember the options and
     // the strings used previously.
@@ -110,9 +112,6 @@ public:
     bool processEndKey(QKeyEvent *event);
     bool processPriorKey(QKeyEvent *event);
     bool processNextKey(QKeyEvent *event);
-    void processDeleteKey(QKeyEvent *event);
-    void processF2Key(QKeyEvent *event);
-    void processF4Key(QKeyEvent *event);
     void processOtherKey(QKeyEvent *event);
     bool processControlArrowKey(QKeyEvent *event);
     bool formatKeyPress(QKeyEvent *event);
