@@ -2,6 +2,7 @@
    Copyright (C) 1998, 1999 Reginald Stadlbauer <reggie@kde.org>
    Copyright (C) 2002-2006 David Faure <faure@kde.org>
    Copyright (C) 2005-2006 Thomas Zander <zander@kde.org>
+   Copyright (C) 2010 Boudewijn Rempt <boud@kogmbh.com>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -25,22 +26,26 @@
 #include "KWDocument.h"
 #include "kword_export.h"
 
-#include <KoCanvasBase.h>
+#include <KWCanvasBase.h>
+
+#include "KWViewMode.h"
 
 #include <QWidget>
 
+class QRect;
+class QPainter;
+
 class KWGui;
 class KWView;
-class KWViewMode;
 class KoToolProxy;
 
+
 /**
- * Class: KWCanvas
  * This class is responsible for the rendering of the frames to
  * the screen as well as the interaction with the user via mouse
  * and keyboard. There is one per view.
  */
-class KWORD_TEST_EXPORT KWCanvas : public QWidget, public KoCanvasBase
+class KWORD_TEST_EXPORT KWCanvas : public QWidget, public KWCanvasBase
 {
     Q_OBJECT
 
@@ -53,7 +58,7 @@ public:
      * @param view the parent KWView object
      * @param parent the parent widget.
      */
-    KWCanvas(const QString& viewMode, KWDocument *document, KWView *view, KWGui *parent);
+    KWCanvas(const QString &viewMode, KWDocument *document, KWView *view, KWGui *parent);
     virtual ~KWCanvas();
 
     /// ask the widget to set the size this canvas takes to display all content
@@ -61,46 +66,23 @@ public:
 
     // KoCanvasBase interface methods.
     /// reimplemented method from superclass
-    virtual void gridSize(qreal *horizontal, qreal *vertical) const;
-    /// reimplemented method from superclass
     virtual bool snapToGrid() const;
+
     /// reimplemented method from superclass
-    virtual void addCommand(QUndoCommand *command);
-    /// reimplemented method from superclass
-    virtual KoShapeManager *shapeManager() const {
-        return m_shapeManager;
-    }
-    /// reimplemented method from superclass
-    virtual void updateCanvas(const QRectF& rc);
-    /// reimplemented method from superclass
-    virtual const KoViewConverter *viewConverter() const;
-    /// reimplemented method from superclass
-    virtual QWidget* canvasWidget() {
+    virtual QWidget *canvasWidget() {
         return this;
     }
     /// reimplemented method from superclass
-    virtual KoUnit unit() const {
-        return document()->unit();
-    }
-    /// reimplemented method from superclass
-    virtual KoToolProxy * toolProxy() const {
-        return m_toolProxy;
-    }
-    /// reimplemented method from superclass
-    virtual void clipToDocument(const KoShape *shape, QPointF &move) const;
-    /// reimplemented method from superclass
-    virtual void updateInputMethodInfo();
-    // getters
-    /// return the document that this canvas works on
-    KWDocument *document() const {
-        return m_document;
+    virtual const QWidget *canvasWidget() const {
+        return this;
     }
 
-    /// return the viewMode currently associated with this canvas
-    KWViewMode *viewMode() const {
-        return m_viewMode;
+    KWView *view() {
+        return m_view;
     }
 
+    virtual void setCursor(const QCursor &cursor);
+
 public slots:
     /**
      * sets the document offset in the scrollArea
@@ -116,7 +98,7 @@ signals:
      */
     void documentSize(const QSizeF &size);
 
-protected:
+protected: // QWidget
     /// reimplemented method from superclass
     virtual void keyPressEvent(QKeyEvent *e);
     /// reimplemented method from superclass
@@ -140,19 +122,17 @@ protected:
     /// reimplemented method from superclass
     virtual void inputMethodEvent(QInputMethodEvent *event);
     /// reimplemented method from superclass
-    virtual void focusOutEvent(QFocusEvent *event);
+    virtual void updateInputMethodInfo();
+    /// reimplemented method from superclass
+    virtual void updateCanvasInternal(const QRectF &clip) { update(clip.toRect()); }
 
 private slots:
     /// Called whenever there was a page added/removed or simply resized.
     void pageSetupChanged();
 
 private:
-    KWDocument *m_document;
-    KoShapeManager *m_shapeManager;
-    KoToolProxy * m_toolProxy;
+
     KWView *m_view;
-    KWViewMode *m_viewMode;
-    QPoint m_documentOffset;
 };
 
 #endif
