@@ -19,16 +19,15 @@
 #ifndef KOLINEBORDERFACTORY_H
 #define KOLINEBORDERFACTORY_H
 
-#include "KoShapeBorderFactory.h"
+#include "KoShapeBorderFactoryBase.h"
 
-class KoLineBorderFactory : public KoShapeBorderFactory
+class KoLineBorderFactory : public KoShapeBorderFactoryBase
 {
 public:
     KoLineBorderFactory(QObject *parent);
 
     /// reimplemented
-    virtual KoShapeBorderModel * createBorder() const;
-
+    virtual KoShapeBorderModel *createBorder() const;
 };
 
 #endif
