@@ -21,6 +21,7 @@
 #include "KoPADocument.h"
 
 #include <KoStore.h>
+#include <KoResourceManager.h>
 #include <KoXmlWriter.h>
 #include <KoXmlReader.h>
 #include <KoOdfStylesReader.h>
@@ -40,7 +41,6 @@
 #include <KoPathShape.h>
 #include <KoLineBorder.h>
 #include <KoXmlNS.h>
-#include <KoDataCenter.h>
 
 #include "KoPACanvas.h"
 #include "KoPAView.h"
@@ -49,6 +49,7 @@
 #include "KoPASavingContext.h"
 #include "KoPALoadingContext.h"
 #include "KoPAViewMode.h"
+#include "KoPAPageProvider.h"
 #include "commands/KoPAPageDeleteCommand.h"
 
 #include <kdebug.h>
@@ -64,7 +65,7 @@ public:
     QList<KoPAPageBase*> masterPages;
     KoInlineTextObjectManager *inlineTextObjectManager;
     bool rulersVisible;
-    QMap<QString, KoDataCenter *>  dataCenterMap;
+    KoPAPageProvider *pageProvider;
 };
 
 KoPADocument::KoPADocument( QWidget* parentWidget, QObject* parent, bool singleViewMode )
@@ -72,14 +73,14 @@ KoPADocument::KoPADocument( QWidget* parentWidget, QObject* parent, bool singleV
     d(new Private())
 {
     d->inlineTextObjectManager = new KoInlineTextObjectManager(this);
+    d->rulersVisible = false;
 
-    // Ask every shapefactory to populate the dataCenterMap
-    foreach(const QString & id, KoShapeRegistry::instance()->keys())
-    {
-        KoShapeFactory *shapeFactory = KoShapeRegistry::instance()->value(id);
-        shapeFactory->populateDataCenterMap(d->dataCenterMap);
-    }
-
+    resourceManager()->setUndoStack(undoStack());
+    resourceManager()->setOdfDocument(this);
+    QVariant variant;
+    d->pageProvider = new KoPAPageProvider();
+    variant.setValue<void*>(d->pageProvider);
+    resourceManager()->setResource(KoText::PageProvider, variant);
     loadConfig();
 }
 
@@ -88,7 +89,7 @@ KoPADocument::~KoPADocument()
     saveConfig();
     qDeleteAll( d->pages );
     qDeleteAll( d->masterPages );
-    qDeleteAll( d->dataCenterMap );
+    delete d->pageProvider;
     delete d;
 }
 
@@ -96,7 +97,7 @@ void KoPADocument::paintContent( QPainter &painter, const QRect &rect)
 {
     KoPAPageBase * page = pageByIndex( 0, false );
     Q_ASSERT( page );
-    QPixmap thumbnail = page->thumbnail( rect.size() );
+    QPixmap thumbnail( pageThumbnail( page, rect.size() ) );
     painter.drawPixmap( rect, thumbnail );
 }
 
@@ -113,32 +114,44 @@ bool KoPADocument::loadOdf( KoOdfReadStore & odfStore )
 {
     emit sigProgress( 0 );
     KoOdfLoadingContext loadingContext( odfStore.styles(), odfStore.store(), componentData() );
-    KoPALoadingContext paContext( loadingContext, dataCenterMap() );
+    KoPALoadingContext paContext(loadingContext, resourceManager());
 
     KoXmlElement content = odfStore.contentDoc().documentElement();
     KoXmlElement realBody ( KoXml::namedItemNS( content, KoXmlNS::office, "body" ) );
 
     if ( realBody.isNull() ) {
-        kError() << "No body tag found!" << endl;
+        kError(30010) << "No body tag found!" << endl;
         return false;
     }
 
     KoXmlElement body = KoXml::namedItemNS(realBody, KoXmlNS::office, odfTagName( false ));
 
     if ( body.isNull() ) {
-        kError() << "No office:" << odfTagName( false ) << " tag found!" << endl;
+        kError(30010) << "No office:" << odfTagName( false ) << " tag found!" << endl;
         return false;
     }
 
     // Load text styles before the corresponding text shapes try to use them!
     KoTextSharedLoadingData * sharedData = new KoTextSharedLoadingData();
-    KoStyleManager * styleManager = dynamic_cast<KoStyleManager *>( dataCenterMap()["StyleManager"] );
-    sharedData->loadOdfStyles(loadingContext, styleManager);
     paContext.addSharedData( KOTEXT_SHARED_LOADING_ID, sharedData );
+    KoStyleManager *styleManager = resourceManager()->resource(KoText::StyleManager).value<KoStyleManager*>();
+
+    sharedData->loadOdfStyles(paContext, styleManager);
 
     d->masterPages = loadOdfMasterPages( odfStore.styles().masterPages(), paContext );
+    if ( !loadOdfProlog( body, paContext ) ) {
+        return false;
+    }
     d->pages = loadOdfPages( body, paContext );
 
+    // create pages if there are none
+    if (d->masterPages.empty()) {
+        d->masterPages.append(newMasterPage());
+    }
+    if (d->pages.empty()) {
+        d->pages.append(newPage(static_cast<KoPAMasterPage*>(d->masterPages.first())));
+    }
+
     if ( !loadOdfEpilogue( body, paContext ) ) {
         return false;
     }
@@ -149,20 +162,12 @@ bool KoPADocument::loadOdf( KoOdfReadStore & odfStore )
         setActionEnabled( KoPAView::ActionDeletePage, false );
     }
 
+    updatePageCount();
+
     emit sigProgress( -1 );
     return true;
 }
 
-bool KoPADocument::completeLoading( KoStore* store )
-{
-    bool ok=true;
-    foreach(KoDataCenter *dataCenter, d->dataCenterMap)
-    {
-        ok = ok && dataCenter->completeLoading(store);
-    }
-    return ok;
-}
-
 bool KoPADocument::saveOdf( SavingContext & documentContext )
 {
     KoXmlWriter* contentWriter = documentContext.odfStore.contentWriter();
@@ -172,13 +177,17 @@ bool KoPADocument::saveOdf( SavingContext & documentContext )
     KoGenStyles mainStyles;
     KoXmlWriter * bodyWriter = documentContext.odfStore.bodyWriter();
 
-    KoPASavingContext paContext( *bodyWriter, mainStyles, documentContext.embeddedSaver, 1, KoShapeSavingContext::Store );
+    KoPASavingContext paContext(*bodyWriter, mainStyles, documentContext.embeddedSaver, 1);
 
     saveOdfDocumentStyles( paContext );
 
     bodyWriter->startElement( "office:body" );
     bodyWriter->startElement( odfTagName( true ) );
 
+    if ( !saveOdfProlog( paContext ) ) {
+        return false;
+    }
+
     if ( !saveOdfPages( paContext, d->pages, d->masterPages ) ) {
         return false;
     }
@@ -190,7 +199,7 @@ bool KoPADocument::saveOdf( SavingContext & documentContext )
     bodyWriter->endElement(); // office:odfTagName()
     bodyWriter->endElement(); // office:body
 
-    mainStyles.saveOdfAutomaticStyles( contentWriter, false );
+    mainStyles.saveOdfStyles( KoGenStyles::DocumentAutomaticStyles, contentWriter );
 
     documentContext.odfStore.closeContentWriter();
 
@@ -219,12 +228,6 @@ bool KoPADocument::saveOdf( SavingContext & documentContext )
     return paContext.saveDataCenter( documentContext.odfStore.store(), documentContext.odfStore.manifestWriter() );
 }
 
-bool KoPADocument::completeSaving( KoStore* store)
-{
-    Q_UNUSED(store)
-    return true;
-}
-
 QList<KoPAPageBase *> KoPADocument::loadOdfMasterPages( const QHash<QString, KoXmlElement*> masterStyles, KoPALoadingContext & context )
 {
     context.odfLoadingContext().setUseStylesAutoStyles( true );
@@ -245,12 +248,16 @@ QList<KoPAPageBase *> KoPADocument::loadOdfMasterPages( const QHash<QString, KoX
 
 QList<KoPAPageBase *> KoPADocument::loadOdfPages( const KoXmlElement & body, KoPALoadingContext & context )
 {
+    if (d->masterPages.isEmpty()) { // we require at least one master page. Auto create one if the doc was faulty.
+        d->masterPages << newMasterPage();
+    }
+
     QList<KoPAPageBase *> pages;
     KoXmlElement element;
     forEachElement( element, body )
     {
         if ( element.tagName() == "page" && element.namespaceURI() == KoXmlNS::draw ) {
-            KoPAPage* page = newPage();
+            KoPAPage *page = newPage(static_cast<KoPAMasterPage*>(d->masterPages.first()));
             page->loadOdf( element, context );
             pages.append( page );
         }
@@ -265,6 +272,13 @@ bool KoPADocument::loadOdfEpilogue( const KoXmlElement & body, KoPALoadingContex
     return true;
 }
 
+bool KoPADocument::loadOdfProlog( const KoXmlElement & body, KoPALoadingContext & context )
+{
+    Q_UNUSED( body );
+    Q_UNUSED( context );
+    return true;
+}
+
 bool KoPADocument::saveOdfPages( KoPASavingContext &paContext, QList<KoPAPageBase *> &pages, QList<KoPAPageBase *> &masterPages )
 {
     paContext.addOption( KoPASavingContext::DrawId );
@@ -289,6 +303,12 @@ bool KoPADocument::saveOdfPages( KoPASavingContext &paContext, QList<KoPAPageBas
     return true;
 }
 
+bool KoPADocument::saveOdfProlog( KoPASavingContext & paContext )
+{
+    Q_UNUSED( paContext );
+    return true;
+}
+
 bool KoPADocument::saveOdfEpilogue( KoPASavingContext & paContext )
 {
     Q_UNUSED( paContext );
@@ -307,7 +327,7 @@ bool KoPADocument::saveOdfSettings( KoStore * store )
     settingsWriter->startElement("config:config-item-set");
     settingsWriter->addAttribute("config:name", "view-settings");
 
-    KoUnit::saveOdf( settingsWriter, unit() );
+    saveUnitOdf(settingsWriter);
 
     settingsWriter->endElement(); // config:config-item-set
 
@@ -353,7 +373,7 @@ void KoPADocument::loadOdfSettings(  const KoXmlDocument & settingsDoc )
 
 void KoPADocument::saveOdfDocumentStyles( KoPASavingContext & context )
 {
-    KoStyleManager *styleManager = dynamic_cast<KoStyleManager *>( dataCenterMap()["StyleManager"] );
+    KoStyleManager *styleManager = resourceManager()->resource(KoText::StyleManager).value<KoStyleManager*>();
     Q_ASSERT( styleManager );
     styleManager->saveOdf( context.mainStyles() );
 }
@@ -489,11 +509,6 @@ void KoPADocument::pageRemoved( KoPAPageBase * page, QUndoCommand * parent )
     Q_UNUSED( parent );
 }
 
-QMap<QString, KoDataCenter *> KoPADocument::dataCenterMap() const
-{
-    return d->dataCenterMap;
-}
-
 KoPAPageBase * KoPADocument::pageByShape( KoShape * shape ) const
 {
     KoShape * parent = shape;
@@ -511,8 +526,16 @@ void KoPADocument::updateViews(KoPAPageBase *page)
 
     foreach (KoView *view, views()) {
         KoPAView *paView = static_cast<KoPAView *>(view);
-        if (paView->activePage() == page)
-            paView->updateActivePage(page);
+        if ( paView->activePage() == page ) {
+            paView->updateActivePage( page );
+        }
+        else if ( dynamic_cast<KoPAMasterPage *>( page ) ) {
+            // if the page changed is a master page, we need to check whether it is the current page's master page
+            KoPAPage *activePage = dynamic_cast<KoPAPage *>( paView->activePage() );
+            if ( activePage && activePage->masterPage() == page ) {
+                paView->updateActivePage( activePage );
+            }
+        }
     }
 }
 
@@ -521,6 +544,24 @@ KoPageApp::PageType KoPADocument::pageType() const
     return KoPageApp::Page;
 }
 
+QPixmap KoPADocument::pageThumbnail(KoPAPageBase* page, const QSize& size)
+{
+    int pageNumber = pageIndex(page) + 1;
+    d->pageProvider->setPageData(pageNumber, page);
+    return page->thumbnail(size);
+}
+
+void KoPADocument::initEmpty()
+{
+    d->masterPages.clear();
+    d->pages.clear();
+    KoPAMasterPage * masterPage = newMasterPage();
+    d->masterPages.append( masterPage );
+    KoPAPage * page = newPage( masterPage );
+    d->pages.append( page );
+    KoDocument::initEmpty();
+}
+
 void KoPADocument::setActionEnabled( int actions, bool enable )
 {
     foreach( KoView *view, views() )
@@ -543,6 +584,7 @@ void KoPADocument::insertPage( KoPAPageBase* page, int index )
     }
 
     pages.insert( index, page );
+    updatePageCount();
 
     setActionEnabled( KoPAView::ActionDeletePage, pages.size() > 1 );
 
@@ -568,6 +610,7 @@ void KoPADocument::insertPage( KoPAPageBase* page, KoPAPageBase* after )
     }
 
     pages.insert( index, page );
+    updatePageCount();
 
     setActionEnabled( KoPAView::ActionDeletePage, pages.size() > 1 );
 
@@ -598,6 +641,7 @@ int KoPADocument::takePage( KoPAPageBase *page )
                 kopaView->updateActivePage( newActivePage );
             }
         }
+        updatePageCount();
     }
 
     if ( pages.size() == 1 ) {
@@ -711,15 +755,18 @@ bool KoPADocument::rulersVisible() const
     return d->rulersVisible;
 }
 
-
 int KoPADocument::pageCount() const
 {
     return d->pages.count();
 }
 
-void KoPADocument::insertIntoDataCenterMap(QString key, KoDataCenter *dc)
+void KoPADocument::updatePageCount()
 {
-    d->dataCenterMap[key] = dc;
+    if (resourceManager()->hasResource(KoText::InlineTextObjectManager)) {
+        QVariant var = resourceManager()->resource(KoText::InlineTextObjectManager);
+        KoInlineTextObjectManager *om = var.value<KoInlineTextObjectManager*>();
+        om->setProperty( KoInlineObject::PageCount, pageCount() );
+    }
 }
 
-#include "KoPADocument.moc"
+#include <KoPADocument.moc>
