@@ -1,5 +1,5 @@
 /* This file is part of the KDE project
-   Copyright (C) 2006-2008 Thorsten Zachmann <zachmann@kde.org>
+   Copyright (C) 2006-2010 Thorsten Zachmann <zachmann@kde.org>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -75,13 +75,6 @@ public:
      */
     QList<KoPAPageBase *> loadOdfPages( const KoXmlElement & body, KoPALoadingContext & context );
 
-    /// reimplemented
-    virtual bool completeLoading( KoStore* store );
-
-    /// reimplemented
-    virtual bool completeSaving( KoStore* store );
-
-
     /**
      * Save pages
      *
@@ -134,7 +127,7 @@ public:
     /**
      * Insert page to the document at index
      *
-     * The function checks if it is a normal or a master page and puts it in 
+     * The function checks if it is a normal or a master page and puts it in
      * the correct list.
      *
      * @param page to insert to document
@@ -145,7 +138,7 @@ public:
     /**
      * Insert @p page to the document after page @p before
      *
-     * The function checks if it is a normal or a master page and puts it in 
+     * The function checks if it is a normal or a master page and puts it in
      * the correct list.
      *
      * @param page to insert to document
@@ -180,16 +173,16 @@ public:
      *
      * The page is created with new.
      *
-     * Reimplement when you need a derivered class in your kopageapplication 
+     * Reimplement when you need a derivered class in your kopageapplication
      */
-    virtual KoPAPage * newPage( KoPAMasterPage * masterPage = 0 );
+    virtual KoPAPage *newPage(KoPAMasterPage *masterPage);
 
     /**
      * Get a new master page for inserting into the document
      *
      * The page is created with new.
      *
-     * Reimplement when you need a derivered class in your kopageapplication 
+     * Reimplement when you need a derivered class in your kopageapplication
      */
     virtual KoPAMasterPage * newMasterPage();
 
@@ -205,8 +198,6 @@ public:
     void setRulersVisible(bool visible);
     bool rulersVisible() const;
 
-    virtual QMap<QString, KoDataCenter *>  dataCenterMap() const;
-
     /**
      * Get the page on which the shape is located
      *
@@ -229,6 +220,13 @@ public:
      */
     virtual KoPageApp::PageType pageType() const;
 
+    /**
+     * Get the thumbnail for the page.
+     *
+     * Use this method instead the one in the pages directly
+     */
+    QPixmap pageThumbnail(KoPAPageBase* page, const QSize& size);
+
 public slots:
     /// reimplemented
     virtual void initEmpty();
@@ -240,10 +238,16 @@ signals:
     void pageRemoved(KoPAPageBase* page);
 
 protected:
-    void insertIntoDataCenterMap(QString key, KoDataCenter *dc);
-
     virtual KoView *createViewInstance( QWidget *parent ) = 0;
 
+    /**
+     * Load the presentation declaration
+     *
+     * The default implementation is empty
+     */
+    virtual bool loadOdfProlog( const KoXmlElement & body, KoPALoadingContext & context );
+
+
     /**
      * Load the epilouge
      *
@@ -251,6 +255,13 @@ protected:
      */
     virtual bool loadOdfEpilogue( const KoXmlElement & body, KoPALoadingContext & context );
 
+    /**
+     * Save the prolog
+     *
+     * The default implementation is empty
+     */
+    virtual bool saveOdfProlog( KoPASavingContext & paContext );
+
     /**
      * Save the epilouge
      *
@@ -269,7 +280,7 @@ protected:
     void loadOdfSettings( const KoXmlDocument & settingsDoc );
 
     /**
-     * This function is called by at the end of addShape. This is used 
+     * This function is called by at the end of addShape. This is used
      * e.g. for doing work on the application which is in the KoShapeAppData.
      *
      * The default impementation does nothing
@@ -277,7 +288,7 @@ protected:
     virtual void postAddShape( KoPAPageBase * page, KoShape * shape );
 
     /**
-     * This function is called by at the end of removeShape. This is used 
+     * This function is called by at the end of removeShape. This is used
      * e.g. for doing work on the application which is in the KoShapeAppData.
      *
      * The default impementation does nothing
@@ -309,6 +320,9 @@ protected:
     /// Save the configuration
     void saveConfig();
 
+    /// set the page count so it gets shown correctly in variables
+    void updatePageCount();
+
 private:
     class Private;
     Private * const d;
