@@ -3,7 +3,8 @@
    Copyright (C) 2006-2009 Thorsten Zachmann <zachmann@kde.org>
    Copyright (C) 2007 Thomas Zander <zander@kde.org>
    Copyright (C) 2009 Inge Wallin   <inge@lysator.liu.se>
-
+   Copyright (C) 2010 Boudewijn Rempt <boud@kogmbh.com>
+   
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
@@ -30,7 +31,7 @@
 #include <QClipboard>
 #include <QLabel>
 
-#include <KoCanvasController.h>
+#include <KoCanvasControllerWidget.h>
 #include <KoResourceManager.h>
 #include <KoColorBackground.h>
 #include <KoFind.h>
@@ -181,7 +182,8 @@ void KoPAView::initGUI()
     setLayout( gridLayout );
 
     d->canvas = new KoPACanvas( this, d->doc );
-    d->canvasController = new KoCanvasController( this );
+    KoCanvasControllerWidget *canvasController = new KoCanvasControllerWidget( this );
+    d->canvasController = canvasController;
     d->canvasController->setCanvas( d->canvas );
     KoToolManager::instance()->addController( d->canvasController );
     KoToolManager::instance()->registerTools( actionCollection(), d->canvasController );
@@ -224,15 +226,15 @@ void KoPAView::initGUI()
 
     gridLayout->addWidget(d->horizontalRuler, 0, 1);
     gridLayout->addWidget(d->verticalRuler, 1, 0);
-    gridLayout->addWidget( d->canvasController, 1, 1 );
+    gridLayout->addWidget(canvasController, 1, 1 );
 
-    connect(d->canvasController, SIGNAL(canvasOffsetXChanged(int)),
+    connect(d->canvasController->proxyObject, SIGNAL(canvasOffsetXChanged(int)),
             this, SLOT(pageOffsetChanged()));
-    connect(d->canvasController, SIGNAL(canvasOffsetYChanged(int)),
+    connect(d->canvasController->proxyObject, SIGNAL(canvasOffsetYChanged(int)),
             this, SLOT(pageOffsetChanged()));
-    connect(d->canvasController, SIGNAL(sizeChanged(const QSize&)),
+    connect(d->canvasController->proxyObject, SIGNAL(sizeChanged(const QSize&)),
             this, SLOT(pageOffsetChanged()));
-    connect(d->canvasController, SIGNAL(canvasMousePositionChanged(const QPoint&)),
+    connect(d->canvasController->proxyObject, SIGNAL(canvasMousePositionChanged(const QPoint&)),
             this, SLOT(updateMousePosition(const QPoint&)));
     d->verticalRuler->createGuideToolConnection(d->canvas);
     d->horizontalRuler->createGuideToolConnection(d->canvas);
@@ -241,15 +243,13 @@ void KoPAView::initGUI()
     if (shell())
     {
         shell()->createDockWidget( &toolBoxFactory );
-
-        connect( d->canvasController, SIGNAL( toolOptionWidgetsChanged(const QMap<QString, QWidget *> &, QWidget*) ),
+        connect( canvasController, SIGNAL( toolOptionWidgetsChanged(const QMap<QString, QWidget *> &, QWidget*) ),
              shell()->dockerManager(), SLOT( newOptionWidgets(const  QMap<QString, QWidget *> &, QWidget*) ) );
     }
 
     connect(shapeManager(), SIGNAL(selectionChanged()), this, SLOT(selectionChanged()));
-    connect(d->canvas, SIGNAL(documentSize(const QSize&)), d->canvasController, SLOT(setDocumentSize(const QSize&)));
-    connect(d->canvasController, SIGNAL(moveDocumentOffset(const QPoint&)),
-            d->canvas, SLOT(setDocumentOffset(const QPoint&)));
+    connect(d->canvas, SIGNAL(documentSize(const QSize&)), d->canvasController->proxyObject, SLOT(updateDocumentSize(const QSize&)));
+    connect(d->canvasController->proxyObject, SIGNAL(moveDocumentOffset(const QPoint&)), d->canvas, SLOT(setDocumentOffset(const QPoint&)));
 
     if (shell()) {
         KoPADocumentStructureDockerFactory structureDockerFactory( KoDocumentSectionView::ThumbnailMode, d->doc->pageType() );
@@ -281,7 +281,7 @@ void KoPAView::initActions()
     d->deleteSelectionAction->setShortcut(QKeySequence("Del"));
     d->deleteSelectionAction->setEnabled(false);
     connect(d->deleteSelectionAction, SIGNAL(triggered()), this, SLOT(editDeleteSelection()));
-    connect(d->canvas->toolProxy(),    SIGNAL(selectionChanged(bool)), 
+    connect(d->canvas->toolProxy(),    SIGNAL(selectionChanged(bool)),
             d->deleteSelectionAction, SLOT(setEnabled(bool)));
 
     KToggleAction *showGrid= d->doc->gridData().gridToggleAction(d->canvas);
@@ -570,7 +570,7 @@ void KoPAView::slotZoomChanged( KoZoomMode::Mode mode, qreal zoom )
         if (mode == KoZoomMode::ZOOM_PAGE) {
             KoPageLayout &layout = d->activePage->pageLayout();
             QRectF pageRect( 0, 0, layout.width, layout.height );
-            d->canvasController->ensureVisible( pageRect );
+            d->canvasController->ensureVisible(d->canvas->viewConverter()->documentToView(pageRect));
         } else if (mode == KoZoomMode::ZOOM_WIDTH) {
             // horizontally center the page
             KoPageLayout &layout = d->activePage->pageLayout();
@@ -716,7 +716,7 @@ void KoPAView::setActivePage( KoPAPageBase* page )
     if ( shell() && pageChanged ) {
         d->documentStructureDocker->setActivePage(d->activePage);
     }
-    
+
     // Set the current page number in the canvas resource provider
     d->canvas->resourceManager()->setResource( KoCanvasResource::CurrentPage, d->doc->pageIndex(d->activePage)+1 );
 }
