@@ -637,8 +637,24 @@ void KoTextLoader::loadList(const KoXmlElement &element, QTextCursor &cursor, bo
             loadBody(e, cursor, isDeleteChange);
         }
 
-        if (!current.textList())
+        if (!current.textList()) {
+            if (!d->currentList->style()->hasLevelProperties(level)) {
+                KoListLevelProperties llp;
+                // Look if one of the lower levels are defined to we can copy over that level.
+                for(int i = level - 1; i >= 0; --i) {
+                    if(d->currentList->style()->hasLevelProperties(i)) {
+                        llp = d->currentList->style()->levelProperties(i);
+                        break;
+                    }
+                }
+                llp.setLevel(level);
+                // TODO make the 10 configurable
+                llp.setIndent(level * 10.0);
+                d->currentList->style()->setLevelProperties(llp);
+            }
+
             d->currentList->add(current, level);
+        }
 
         if (listHeader)
             blockFormat.setProperty(KoParagraphStyle::IsListHeader, true);
@@ -1167,7 +1183,7 @@ void KoTextLoader::loadTable(const KoXmlElement &tableElem, QTextCursor &cursor)
                     if (!columnStyleName.isEmpty()) {
                         KoTableColumnStyle *columnStyle = d->textSharedData->tableColumnStyle(columnStyleName, d->stylesDotXml);
                         for (int c = columns; c < columns + repeatColumn; c++) {
-                            tcarManager->setColumnStyle(c, columnStyle);
+                            tcarManager->setColumnStyle(c, *columnStyle);
                         }
                     }
                     columns = columns + repeatColumn;
@@ -1179,7 +1195,7 @@ void KoTextLoader::loadTable(const KoXmlElement &tableElem, QTextCursor &cursor)
                     QString rowStyleName = tblTag.attributeNS(KoXmlNS::table, "style-name", "");
                     if (!rowStyleName.isEmpty()) {
                         KoTableRowStyle *rowStyle = d->textSharedData->tableRowStyle(rowStyleName, d->stylesDotXml);
-                        tcarManager->setRowStyle(rows, rowStyle);
+                        tcarManager->setRowStyle(rows, *rowStyle);
                     }
                     rows++;
                     if (columns > 0)
