@@ -2,6 +2,7 @@
  * Copyright (C) 1998, 1999 Torben Weis <weis@kde.org>
  * Copyright (C) 2000-2005 David Faure <faure@kde.org>
  * Copyright (C) 2007-2008 Thorsten Zachmann <zachmann@kde.org>
+ * Copyright (C) 2010 Boudewijn Rempt <boud@kogmbh.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -28,6 +29,8 @@
 #include "KoMainWindow.h"
 #include "KoFilterManager.h"
 #include "KoDocumentInfo.h"
+#include "KoCanvasController.h"
+#include "KoCanvasControllerWidget.h"
 #include "rdf/KoDocumentRdfBase.h"
 #ifdef SHOULD_BUILD_RDF
 #include "rdf/KoDocumentRdf.h"
@@ -69,6 +72,8 @@
 #include <QtGui/QApplication>
 #include <QtGui/QPrinter>
 #include <QtGui/QPrintDialog>
+#include <QGraphicsScene>
+#include <QGraphicsProxyWidget>
 
 // Define the protocol used here for embedded documents' URL
 // This used to "store" but KUrl didn't like it,
@@ -118,7 +123,9 @@ public:
             storeInternal(false),
             bLoading(false),
             startUpWidget(0),
-            undoStack(0)
+            undoStack(0),
+            canvasItem(0)
+
     {
         confirmNonNativeSave[0] = true;
         confirmNonNativeSave[1] = true;
@@ -191,6 +198,8 @@ public:
     bool bEmpty;
 
     KoPageLayout pageLayout;
+
+    QGraphicsItem *canvasItem;
 };
 
 // Used in singleViewMode
@@ -334,6 +343,7 @@ KoDocument::~KoDocument()
     }
 
     delete d->filterManager;
+
     delete d;
     s_documentList->removeOne(this);
     // last one?
@@ -2488,6 +2498,23 @@ void KoDocument::setEmpty()
     d->bEmpty = true;
 }
 
+QGraphicsItem *KoDocument::canvasItem()
+{
+    if (!d->canvasItem) {
+        d->canvasItem = createCanvasItem();
+    }
+    return d->canvasItem;
+}
+
+QGraphicsItem *KoDocument::createCanvasItem()
+{
+    KoView *view = createView();
+    QGraphicsProxyWidget *proxy = new QGraphicsProxyWidget();
+    QWidget *canvasController = view->findChild<KoCanvasControllerWidget*>();
+    proxy->setWidget(canvasController);
+    return proxy;
+}
+
 // static
 int KoDocument::defaultAutoSave()
 {
