@@ -31,6 +31,7 @@
 #include <KoXmlNS.h>
 #include <KoStoreDevice.h>
 #include <KoUnit.h>
+#include <KoGenStyle.h>
 
 #include <QPainter>
 #include <QTimer>
@@ -64,6 +65,11 @@ void RenderQueue::renderImage()
     }
 }
 
+void RenderQueue::updateShape()
+{
+    m_pictureShape->update();
+}
+
 //////////////
 PictureShape::PictureShape()
     : KoFrameShape(KoXmlNS::draw, "image"),
@@ -141,7 +147,7 @@ void PictureShape::paint(QPainter &painter, const KoViewConverter &converter)
         QTimer::singleShot(0, m_renderQueue, SLOT(renderImage()));
         if (!imageData->hasCachedPixmap()
             || imageData->pixmap().size().width() > pixmapSize.width()) { // don't scale down
-            update(pixelsF); // schedule another update, by then the renderqueue might have rendered the pixmap
+            QTimer::singleShot(0, m_renderQueue, SLOT(updateShape()));
             return;
         }
         pixmap = imageData->pixmap();
@@ -240,6 +246,14 @@ KoImageCollection *PictureShape::imageCollection() const
     return m_imageCollection;
 }
 
+QString PictureShape::saveStyle(KoGenStyle& style, KoShapeSavingContext& context) const
+{
+    if(transparency() > 0.0) {
+        style.addProperty("draw:image-opacity", QString("%1%").arg((1.0 - transparency()) * 100.0));
+    }
+    return KoShape::saveStyle(style, context);
+}
+
 void PictureShape::loadStyle(const KoXmlElement& element, KoShapeLoadingContext& context)
 {
     KoShape::loadStyle(element, context);
@@ -259,6 +273,10 @@ void PictureShape::loadStyle(const KoXmlElement& element, KoShapeLoadingContext&
             setMode(Watermark);
         }
     }
+    const QString opacity(styleStack.property(KoXmlNS::draw, "image-opacity"));
+    if (! opacity.isEmpty() && opacity.right(1) == "%") {
+        setTransparency(1.0 - (opacity.left(opacity.length() - 1).toFloat() / 100.0));
+    }
 }
 
 PictureShape::PictureMode PictureShape::mode() const
